#!/usr/bin/env python3

from astropy.time import Time
import sys

try:
    import epics_fetch
except ImportError as e:
    try:
        from bin import epics_fetch
    except ImportError:
        raise ImportError(f'epics_fetch.py not in PYTHONPATH:\n {sys.path}')
try:
    import tpmdata
    tpmdata.tinit()
except ImportError:
    print('tpmdata unavailable')
    tpmdata = None

__version__ = '3.0.0'


def query():
    if tpmdata is None:
        raise ConnectionError('Cannot query the tpm without tpmdata installed')

    data = tpmdata.packet(1, 1)

    t = Time(data['ctime'], format='unix')
    output = f'Status at:  {t.isot[12:19]}Z\n'
    output += (f"Telescope stowed at:  "
               f"{data['az_actual_pos']*data['az_spt']/3600:>5.1f},"
               f" {data['alt_actual_pos']*data['alt_spt']/3600:>5.1f},"
               f" {data['rot_actual_pos']*data['rot_spt']/3600:>5.1f} mount\n")
    epics_data = epics_fetch.get_data(['25m:mcp:instrumentNum'],
                                      start_time=Time.now().to_datetime(),
                                      end_time=Time.now().to_datetime())
    output += f"Instrument mounted:  {epics_data[0].values[-1]}\n"
    output += (f"Counterweights at:  {data['plc_cw_0']:.1f},"
               f" {data['plc_cw_1']:.1f},"
               f" {data['plc_cw_2']:.1f},"
               f" {data['plc_cw_3']:.1f}\n")
    # TODO Check if this is supposed to do something
    output += f"LN2 autofill systems:  Connected and turned on\n"
    output += (f"180L LN2 dewar scale:  SP1 {data['dewar_sp1_lb']:6.1f} lbs,"
               f" {data['dewar_sp1_psi']:6.1f} psi")

    return output


def main():
    print(query())


if __name__ == '__main__':
    main()



#!/usr/bin/env python
"""A script to parse guider images and grab essential variables from it.
While the rest of this repository is focused on logging, this is mostly
piggybacking off of previous work in order to collect guider data to
identify anomolous lurches in guiding, first identified around 2019-10-30.

2019-10-31:     dgatlin     Init
2019-11-11:     dgatlin     Moved outputs to a separate folder by mjd, made
                                table to help find lurch events.
"""
import numpy as np
import argparse
import glob
import os
import stat

try:
    import sjd
except ImportError as e:
    raise ImportError('Please add ObserverTools/bin to your PYTHONPATH:\n'
                      '    {}'.format(e))

from astropy.io import fits
from astropy.time import Time
# from pathlib import Path

__version__ = '3.0.1'


class GuiderRaw:
    """A class to parse raw data from APOGEE. The purpose of collecting this
    raw data is to future-proof things that need these ouptuts in case
    things like autoschedulers change, which many libraries depend on. This
    will hopefully help SDSS-V logging"""
    def __init__(self, fits_file, ext):
        fil = fits.open(fits_file)
        # layer = self.image[layer_ind]
        header = fil[ext].header
        # An A dither is DITHPIX=12.994, a B dither is DITHPIX=13.499
        self.hdu = fil[ext]
        self.exp_time = header['EXPTIME']
        self.datetimet = Time(header['DATE-OBS'])  # Local
        self.exp_id = int(str(fits_file).split('-')[-1].split('.')[0])
        # self.seeing = header['SEEING'] # Inconsistent
        self.img_type = header['IMAGETYP']
        self.n_read = len(fil)-1
        

def main():
    parser = argparse.ArgumentParser()
    parser.add_argument('-t', '--today', action='store_true')
    parser.add_argument('-m', '--mjd',
                        help='If not today (-t), the mjd to search')
    parser.add_argument('-l', '--lurches', nargs='+', default=[],
                        help='The exposure ids of known lurches')
    parser.add_argument('-v', '--verbose', action='store_true',
                        help='Whether or not you want to print a table')

    args = parser.parse_args()
    if args.today:
        mjd_today = sjd.curSjd()
        data_dir = '/data/gcam/{}/'.format(mjd_today)
        mjd_target = mjd_today
    elif args.mjd:
        data_dir = '/data/gcam/{}/'.format(args.mjd)
        mjd_target = args.mjd
    else:
        raise Exception('Must provide -t or -m in arguments')
    print(data_dir)
    if args.lurches:
        print('Looking for lurches at {}'.format(', '.join(args.lurches)))

    eids = []
    ts = []
    rot = []
    ra = []
    dec = []
    seeing = []
    scale = []
    lurches = []
    files = glob.glob(data_dir + 'proc-gimg*.fits.gz')
    for i, fil in enumerate(files):
        try:
            img = GuiderRaw(fil, 0)
            rot.append(img.hdu.header['DROT'])  # Must be the first one in
            # case there is no DROT, so that it fails before appending
            ra.append(img.hdu.header['DRA'])
            dec.append(img.hdu.header['DDEC'])
            ts.append(Time(img.hdu.header['DATE-OBS']))
            seeing.append(img.hdu.header['SEEING'])
            scale.append(img.hdu.header['SCALE'])
            eids.append(str(int(fil.split('-')[-1].split('.')[0])))
            lurches.append(eids[-1] in args.lurches)
            if i % 100 == 0:
                print('{}/{}'.format(i, len(files)))
        except KeyError:
            pass

    for i, lu in enumerate(lurches):  # in order to display well in the tab
        if lu:
            lurches[i] = 'Y'
        else:
            lurches[i] = ' '
    # There are ridiculous errors on slews (-gazillion), so I need to filter
    # those out. Also, glob.glob is infamously unsorted, so I need to sort
    # what's left. I also need to filter eids before I sort them or I will
    # have index errors
    eids = np.array(eids).astype(int)
    rot = np.array(rot) * 3600
    filt = rot > -1e6
    sorter = eids[filt].argsort()
    eids = eids[filt][sorter]
    ts = np.array(ts)[filt][sorter]
    rot = rot[filt][sorter]
    lurches = np.array(lurches)[filt][sorter]
    ra = np.array(ra)[filt][sorter] * 3600
    dec = np.array(dec)[filt][sorter] * 3600
    seeing = np.array(seeing)[filt][sorter]
    scale = (np.array(scale)[filt][sorter] - 1) * 1e5
    # print(ts.shape, rot.shape, ra.shape, dec.shape)
    
    save_dir = os.path.dirname(os.path.realpath(
        __file__)) + '/guider_issues/{}'.format(mjd_target)
    # print(save_dir)
    t_out = [t.in_time for t in ts]
    os.system('rm -r {}'.format(save_dir))
    os.system('mkdir {}'.format(save_dir))
    np.save(save_dir + '/times.npy', t_out)
    np.save(save_dir + '/rotator.npy', rot)
    np.save(save_dir + '/ra.npy', ra)
    np.save(save_dir + '/dec.npy', dec)
    np.save(save_dir + '/lurches.npy', np.array(args.lurches).astype(int))
    np.save(save_dir + '/ids.npy', eids)
    
    tab = open(save_dir + '/guiding_table.txt', 'w')
    
    tab.write('# {:<6}  {:<8}  {:<8}  {:<7}  {:<7}  {:<7}  {:<7}'
              '  {:<6}\n'.format('Time', 'Exp ID', "Rot ('')", "RA ('')",
                                 "Dec ('')", "See ('')", 'Scale-1 (e5)',
                                 'Lurch?'))
    if args.verbose:
        print('# {:<6}  {:<8}  {:<8}  {:<8}  {:<7}  {:<7}  {:<7}'
              '  {:<6}'.format('Time', 'Exp ID', "Rot ('')", "RA ('')",
                               "Dec ('')", "See ('')", 'Scale-1 (e5)',
                               'Lurch?'))

    tab.write('#' + '-' * 79 + '\n')
    if args.verbose:
        print('#' + '-' * 79)
    for t, i, r, a, d, s, sc, lu in zip(
            ts, eids, rot, ra, dec, seeing, scale, lurches):
        tab.write('{}  {:<8.0f}  {:<+8.2f}  {:<+8.2f}  {:<+8.2f}  {:<+8.1f}'
                  '  {:<+8.1f} {:<6}\n'.format(t.time, i, r, a, d, s, sc, lu))
        if args.verbose:
            print('{}  {:<8.0f}  {:<+8.2f}  {:<+8.2f}  {:<+8.2f}  {:<+8.1f}'
                  '  {:<+8.1f} {:<6}'.format(t.time, i, r, a, d, s, sc, lu))
    
    os.chmod(save_dir, stat.S_IRWXO)  # If someone else runs this, I need to
    # still have access or I can't use git on the repo. This makes sure I
    # maintain access to every file
    os.chmod(save_dir, 0o777)
    for fil in glob.glob(save_dir + '/*'):
        os.chmod(fil, 0o777)


if __name__ == '__main__':
    main()



#!/usr/bin/env python3
"""
XMID
A tool to test spectrograph collimation using SOS/DOS, a twin to WAVEMID.
EM 09/04/2010

usage:
XMID 4908.6 7984.0 4999.9 8058.3 pro[plot]

The above line doesn't need to be typed and is designed to be copied straight
from SOS/DOS

"""
import matplotlib.pyplot as plt
from pathlib import Path
import numpy as np
import sys


__version__ = '3.0.0'


if not sys.argv[1:2]:
    print("    Usage:  XMID  2036.0 2045.0 2043.0 2044.0  [ct]")
    print("    ct - optional cart number, 0 - sos nominal")
    print("    plot - to plot ")
    sys.exit("Error: program requires at least 4 arguments, exit")

nPar = len(sys.argv)
# print " "

# read sosXmids arguments  
sosXmid = np.array(sys.argv[1:]).astype(float)

# read sosCart & plot options
sosCart = 0
plotQ = False
if nPar >= 6:
    if sys.argv[5] == "plot":
        plotQ = True
    else:
        sosCart = int(sys.argv[5])
if nPar >= 7:
    if sys.argv[6] == "plot":
        plotQ = True
    else:
        sosCart = int(sys.argv[6])
# print "sosCart=", sosCart
# print "plotQ=",plotQ

# check if Kaike's table exist on disk
fPath = Path('.')
# fName="bin/xmid.dat"
# fullName=fPath+"/"+fName
fullName = Path(__file__).absolute().parent.parent / 'dat/xmid.dat'
if not fullName.exists():
    sys.exit("{} was not found, exit".format(fullName.absolute()))


# read Kaike's table file  to string array
file = fullName.open('r')
line = "a"
lineArr = []
while line != "":
    line = file.readline()
    line = line.strip()
    if line != "":
        lineArr.append(line)
file.close()

# convert Kaike's table to np.array
# print "--- Kaike's table for flat's sets (dat/xmid.dat)"
# print "cart   b1      r1      b2      r2"
n = len(lineArr)
tblCart = np.zeros([n])
tblXmid = np.zeros([n, 4])
for i in range(0, n):
    aa = lineArr[i].split()
    tblCart[i] = int(aa[0])
    for j in range(0, 4):
        tblXmid[i, j] = float(aa[j + 1])
    ss = "%2i   %6.1f  %6.1f  %6.1f  %6.1f" % (
        tblCart[i], tblXmid[i, 0], tblXmid[i, 1], tblXmid[i, 2], tblXmid[i, 3])
#    if i == 0: print ss, "(sos nominal)"
#   else: print ss

# search sos cart number in Kaike's table 
iii = np.nonzero(tblCart == sosCart)[0]
# print iii,  len(iii)
if len(iii) == 0:
    sys.exit("Error: no requested cart number in the table, exit")
elif len(iii) > 1:
    sys.exit("Error: more then one cart number found in the table, exit")
else:
    tblInd = iii[0]  # index of table for requested cart number
# print "sosInd=", tblInd
# print "tblCart=", tblCart[tblInd]

# select line for rewuested cart and calulate the difference 
tblXmidC = tblXmid[tblInd, 0:len(sosXmid)]  # requested cart parameters from
# table
difXmidC = sosXmid - tblXmidC  # difference between sos and table for requested
# cart

# print "   Requested nominal set for cart=%2i" % (tblCart[tblInd])
print(" " * 13, "  b1     r1     b2     r2")
print("current     :" + (" {:6.1f}" * len(sosXmid)).format(*sosXmid))
print("nominal {:2.0f}  :".format(tblCart[tblInd])
      + (" {:6.1f}" * len(tblXmidC)).format(*tblXmidC))
print("XMID spec   :" + (" {:6.1f}" * len(difXmidC)).format(*difXmidC))
print("-" * 40)

# -a=-b=c
# boss moveColl spec=sp1 a=-63 b=-63 c=63  # +1
step = 63.0
sp1 = round((difXmidC[0] + difXmidC[1]) / 2.0 * step)
print("boss moveColl spec=sp1 a=%i b=%i c=%i" % (sp1, sp1, -sp1))
try:
    sp2 = round((difXmidC[2] + difXmidC[3]) / 2.0 * step)
    print("boss moveColl spec=sp2 a=%i b=%i c=%i" % (sp2, sp2, -sp2))
except IndexError:
    pass

print("-" * 40)
print("The tolerance is +/- 8 pixels (yellow), +/- 12 pixel (red)")
# print ""

#  figure

# check if to plot
if not plotQ:
    sys.exit()

print("Plotting, close window with plot to exit")

tblXmidN = np.zeros([n, 4])
tblXmidN[:, 0:4] = tblXmid[:, 0:] - tblXmid[0, 0:]  # tbl minus tbl sos nominal

sosXmidN = np.zeros([4])
sosXmidN[:] = sosXmid[:] - tblXmid[0, :]  # sos minus tbl sos nominal

plt.figure(num=None, figsize=(9.5, 4.5), )
# plt.ioff()
xmin = -15
xmax = 15
ymin = -15
ymax = 15
xtol = 8
ytol = 8
xtol1 = 12
ytol1 = 12

plt.subplot(121)
# sp1=plt.plot(tblXmidN[1:,0],tblXmidN[1:,1],"bo")
plt.axis([xmin, xmax, ymin, ymax])
plt.grid(True)
plt.xlabel('b1 - b1(nominal), pix')
plt.ylabel('r1 - r1(nominal), pix')
plt.title('sp1 xmid', size=15)
plt.plot([-xtol, xtol, xtol, -xtol, -xtol],
                     [ytol, ytol, -ytol, -ytol, ytol], color='b', linewidth=0.6)
plt.plot([-xtol1, xtol1, xtol1, -xtol1, -xtol1],
                      [ytol1, ytol1, -ytol1, -ytol1, ytol1], color='r',
                      linewidth=0.6)
plt.plot(sosXmidN[0], sosXmidN[1], 'rs')
plt.annotate("current", [sosXmidN[0] + 0.5, sosXmidN[1] - 0.1], color="r",
             size=13)
plt.plot([xmin, xmax], [ymin, ymax], color='black', linewidth=0.6)
# plt.setp(line3, color='black', linewidth=1.0)
# for i in range(1,n):
#     plt.annotate("%2i"% (tblCart[i]), [tblXmidN[i,0]+0.4,tblXmidN[i,1]-0.1],
#     color="b", size=12)

plt.subplot(122)
# sp2=plt.plot(tblXmidN[0:,2],tblXmidN[0:,3],"bo")
plt.axis([xmin, xmax, ymin, ymax])
plt.grid(True)
plt.xlabel('b2 - b2(nominal), pix')
plt.ylabel('r2 - r2(nominal), pix')
plt.title('sp2 xmid', size=15)
plt.plot([-xtol, xtol, xtol, -xtol, -xtol],
                     [ytol, ytol, -ytol, -ytol, ytol], color='b', linewidth=0.6)
plt.plot([-xtol1, xtol1, xtol1, -xtol1, -xtol1],
                      [ytol1, ytol1, -ytol1, -ytol1, ytol1], color='r',
                      linewidth=0.6)
plt.plot(sosXmidN[2], sosXmidN[3], 'rs')
plt.annotate("current", [sosXmidN[2] + 0.5, sosXmidN[3] - 0.1], color="r",
             size=13)
plt.plot([xmin, xmax], [ymin, ymax], color='black', linewidth=0.6)
# for i in range(1,n):
#     plt.annotate("%2i"% (tblCart[i]), [tblXmidN[i,2]+0.4,tblXmidN[i,3]-0.1],
#     color="b", size=12)

# plt.ion()

# plt.draw()
plt.show()
print("")



#!/usr/bin/env python3
import unittest
from pathlib import Path
from bin import ap_test
import numpy as np


class TestAPTest(unittest.TestCase):
    def setUp(self) -> None:
        self.project = Path(__file__).absolute().parent.parent

        class Dummy(object):
            pass

        self.args = Dummy()
        self.args.sjds = [59011]
        self.args.exps = [[34490027, 34490042, 34490051, 34490056, 34490069]]
        self.args.plot = False
        self.args.legacy = False
        self.args.verbose = True
        master_path = self.project / 'dat/master_dome_flat_1.npy'
        self.master_data = np.load(master_path)

    def test_known_date(self):

        self.atest = ap_test.ApogeeFlat(self.master_data, self.args)
        self.atest.run_inputs()

    def test_directory(self):
        """Checks to see if the directory for new data is available to this
        computer"""
        self.assertTrue(Path('/data/apogee/utr_cdr/').exists()
                        or (Path.home() / 'data/apogee/utr_cdr').exists())

    def test_plotting(self):
        """Tests the plotting routine"""
        self.args.plot = True
        self.atest = ap_test.ApogeeFlat(self.master_data, self.args)
        self.atest.run_inputs()

    def test_legacy(self):
        """Test the legacy version of ap_test"""
        self.args.legacy = True
        self.master_data = np.load(self.project / 'dat/utr_master_flat_21180043.npy')
        self.atest = ap_test.ApogeeFlat(self.master_data, self.args)
        self.atest.run_inputs()


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python3

"""
A terminal tool to access the SDSS EPICS server using channeltelemetry

See this CA library  https://github.com/RobbieClarken/channeltelemetry

Based off of cafetch.py, written by Elena, with a few features removed to limit
 dependencies.

Changelog:
2020-06-08  DG  Ported to Python 3 and ObserverTools, takes a series of keys or
 just one.

"""

from channelarchiver import Archiver
from argparse import ArgumentParser
from astropy.time import Time
import socket

try:
    telemetry = Archiver('http://sdss-telemetry.apo.nmsu.edu/'
                         'telemetry/cgi/ArchiveDataServer.cgi')
    telemetry.scan_archives()
except (socket.gaierror, ConnectionRefusedError):
    try:
        telemetry = Archiver('http://localhost:5080/'
                             'telemetry/cgi/ArchiveDataServer.cgi')
        telemetry.scan_archives()
    except (socket.gaierror, ConnectionRefusedError) as e:
        raise Exception('Cannot access EPICS Server, aborting, you should try\n'
                        'ssh -L 5080:sdss4-telemetry.apo.nmsu.edu:80 observer@'
                        'sdss-gateway.apo.nmsu.edu')

__version__ = '3.1.1'


def get_data(channel, start_time, end_time):
    """ Grabs the channel data. If a list is passed to channel, it will return
    a list of datasets"""

    telemetry.scan_archives()
    data = telemetry.get(channel, start_time, end_time, interpolation='raw')
    if isinstance(data, list):
        for i, dat in enumerate(data):
            try:
                data[i].times = Time(dat.times)
            except ValueError:
                data[i].times = Time(dat.times, format='iso')

    else:
        try:
            data.times = Time(data.times)
        except ValueError:
            data.times = Time(data.times, format='iso')

    return data


def print_data(data):
    print('    Channel:'.format(data.channel))
    print('    Units: {}'.format(data.units))
    print('    {:<30}| {:<30}'.format('Time', 'Value'))
    print('    ' + ('=' * 76))
    for t, v, in zip(data.times, data.values):
        print('    {:<30}| {}'.format(t.isot, v))


def print_datasets(datasets):
    if isinstance(datasets, list):
        for dataset in datasets:
            print_data(dataset)
    else:
        print_data(datasets)


def parse_args():
    now = Time.now() + 0.3
    parser = ArgumentParser(description='A command line interface for the SDSS'
                                        ' EPICS Server. Prints a simple table'
                                        ' of data. If no time window is'
                                        ' specified, it will print the most'
                                        ' recent value only.')

    parser.add_argument('-c', '--channels', nargs='?',
                        default="25m:mcp:cwPositions",
                        help='A list of channel names, default is'
                             ' 25m:mcp:cwPositions')
    parser.add_argument('--t1', '--startTime', dest='startTime',
                        default=now.isot,
                        help='start time of query, default is 5 min ago, format'
                             ' "2015-06-23 22:10"')
    parser.add_argument('--t2', '--endTime', dest='endTime', default=now.isot,
                        help='end time of query, default is current time now,'
                             ' format "2015-06-23 22:15"')

    args = parser.parse_args()
    return args


def main():
    args = parse_args()
    print(args.channels)
    datasets = get_data(args.channels, args.startTime, args.endTime)
    print_datasets(datasets)


if __name__ == '__main__':
    main()



#!/usr/bin/env python3
import unittest
import subprocess as sub
from bin import boss_sha1sum
from pathlib import Path


class TestBOSSSHA1Sum(unittest.TestCase):

    def test_find_sha1sum(self):
        """Checks to see if sha1sum is installed"""
        x = sub.check_output('which sha1sum', shell=True)
        self.assertTrue(x)

    def test_here(self):
        """Runs it on 3 boss images that are in this directory"""
        here = Path(__file__).parent
        boss_sha1sum.write_hashes(here, here / 'boss.sha1sum')
        (here / 'boss.sha1sum').unlink()

    def test_check_unique(self):
        """This ensures that test_here creates unique hashes"""
        here = Path(__file__).parent
        boss_sha1sum.write_hashes(here, here / 'boss.sha1sum')
        hashes_file = here / 'boss.sha1sum'
        with hashes_file.open('r')as fil:
            hashes_list = []
            for line in fil:
                hsh = line.split()[0]
                self.assertNotIn(hsh, hashes_list)
                hashes_list.append(hsh)

    def inactive_test_old(self):
        """This is how it used to be written, and can be used a benchmark, this
        should be slower"""
        mjd_known = 59009
        cmd = 'sha1sum sdR-*.fit.gz > {}.sha1sum'.format(mjd_known)
        sub.call(cmd, shell=True)
        sub.call('rm {}.sha1sum'.format(mjd_known), shell=True)


if __name__ == '__main__':
    unittest.main()




#!/usr/bin/env python3
import unittest
import sjd
from astropy.time import Time


class TestSJD(unittest.TestCase):

    def test_astropy(self):
        """Checks to see if astropy.time agrees with mjd.mjd"""
        astropy_mjd = int(Time.now().mjd + 0.3)
        mjd_mjd = sjd.sjd()
        print(astropy_mjd, mjd_mjd)
        self.assertEqual(astropy_mjd, mjd_mjd)


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python3

import gzip
import numpy as np
import matplotlib.pyplot as plt
import seaborn as sns
from matplotlib.dates import DateFormatter
from astropy.time import Time
from astropy.table import Table
from argparse import ArgumentParser
from pathlib import Path


sns.set(style='darkgrid')

__version__ = '3.1.0'


class TPMSJD:
    def __init__(self, sjd:int, channels:iter, verbose=False):
        self.sjd = sjd
        self.channels = channels
        self.verbose = verbose
        print('Searching through {}'.format(sjd))
        uncompressed = Path('/data/logs/mcp/tpm-archive-{}.dat'
                            ''.format(sjd))
        if uncompressed.exists():
            if verbose:
                print('    Reading uncompressed data')
            useful_lines = []
            with uncompressed.open('r') as f:
                for line in f:
                    for channel in self.channels:
                        if channel in line:
                            useful_lines.append(line.split())
            useful_lines = np.array(useful_lines)
            self.data = Table(useful_lines, names=('Time', 'Key', 'Value'),
                    dtype=(float, str, float))
            # self.data = ascii.read(uncompressed,
            #                        names=('Time', 'Key', 'Value'))
        else:
            compressed = uncompressed.parent / (uncompressed.name + '.gz')
            if compressed.exists():
                if verbose:
                    print('    Reading compressed data')
                useful_lines = []
                with gzip.open(compressed, 'rb') as f:
                    for line in f:
                        line = line.decode('utf-8')
                        for channel in self.channels:
                            if channel in line:
                                useful_lines.append(line.split())
                    useful_lines = np.array(useful_lines)
                    self.data = Table(useful_lines,
                                      names=('Time', 'Key', 'Value'))
            else:
                print('    No data found for {}'.format(sjd))
                self.data = None
                return
        self.data['Time'] = Time(self.data['Time'], format='unix')


def parse_args():
    parser = ArgumentParser(description='A tool to query the TPM data archives')
    parser.add_argument('--t1', nargs=1,
                        help='The start time of the query')
    parser.add_argument('--t2', nargs=1,
                        help='The start time of the query')
    parser.add_argument('-m', '--mjd', type=int,
                        help='The mjd to search, can be used as an alternative'
                             ' to --t1 and --t2')
    parser.add_argument('-c', '--channels', nargs='+',
                        help='Channel names to query. For an exhaustive list,'
                             ' run `tpm_feed.py --list-channels`')
    parser.add_argument('-v', '--verbose', action='store_true',
                        help='Prints help text')
    args = parser.parse_args()
    return args


def main(args=parse_args()):
    if args.mjd:
        args.t1 = '{}'.format(Time(args.mjd, format='mjd').isot)
        args.t2 = '{}'.format(Time(args.mjd + 1, format='mjd').isot)
    if args.channels:
        for channel in args.channels:
            if args.verbose:
                print(channel)
            if not args.t1:
                print('No time window given, check -h for info, exiting')
                return
            arg_times = Time([args.t1, args.t2])
            tpmsets = []
            for sjd in range(int(arg_times.mjd[0]),
                             int(arg_times.mjd[1]) + 1):
                tpm = TPMSJD(sjd, args.channels, verbose=args.verbose)
                if tpm.data is None:
                    continue
                window = ((tpm.data['Time'] >= (arg_times[0]))
                          & (tpm.data['Time'] <= (arg_times[1])))
                if args.verbose:
                    print('    {} times for day {} are within the window, out'
                          ' of {} times.'.format(np.sum(window), sjd,
                                                 len(tpm.data)))
                tpm.data = tpm.data[window]
                tpmsets.append(tpm)

            times = []
            values = []
            for tpm in tpmsets:
                filt = tpm.data['Key'] == channel
                times = np.append(times, tpm.data['Time'][filt])
                values = np.append(values, tpm.data['Value'][filt])
            if len(times) == 0:
                print('    No times found')
                continue
            times = Time(times)
            values = np.array(values).astype(float)
            fig = plt.figure(figsize=(6, 4), dpi=100)
            ax = fig.gca()
            ax.plot_date(times.plot_date, values, '-', drawstyle='steps-post')
            ax.set_title(channel)
            ax.set_xlabel('{} to {}'.format(times[0].iso[:16],
                                            times[-1].iso[:16]))
            ax.set_xlim(*(arg_times).plot_date)
            ax.set_xticklabels(ax.get_xticks(), rotation=35)
            ax.xaxis.set_major_formatter(DateFormatter('%RZ'))
            plt.show()
    else:
        print('No channel specified, nothing to do')


if __name__ == '__main__':
    main()



#!/usr/bin/env python3
import unittest
import subprocess as sub
from pathlib import Path


class TestHelp(unittest.TestCase):

    def test_noargs(self):
        """Runs the help like a user normally would"""
        here = Path(__file__).absolute().parent.parent
        sub.call([(here / 'bin/help.py').as_posix(), '-h'])


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python3
import unittest
from pathlib import Path
import time_track


class TestTPMFetch(unittest.TestCase):

    def setUp(self):
        self.file = time_track.__file__

        class DummyArgs(object):
            pass

        self.args = DummyArgs()
        self.args.m1 = 59216
        self.args.m2 = 59246
        self.args.apogee = True
        self.args.bhm = True
        self.args.mwm = True
        self.args.eboss = False
        self.args.manga = False
        self.args.force = True
        self.args.verbose = True

    def test_january_2021(self):
        time_track.main(args=self.args)


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python3
"""
sp_version.py

Outputs a list of software versions used that night. Rewritten by Dylan Gatlin
 based on spVersion by Elena Malanushenko

Changelog:
2020-06-08  DG  Ported to ObserverTools in Python 3, replaced os with sub
"""

import subprocess as sub

__version__ = '3.0.0'


def main():
    softwares, versions = [], []
    idlspec2d = sub.Popen('setup idlspec2d; idlspec2d_version', shell=True,
                          stdout=sub.PIPE)
    softwares.append('idlspec2d')
    versions.append(idlspec2d.stdout.read().decode('utf-8').strip('\n'))

    plate_mapper = sub.Popen('setup idlmapper; echo "print, idlmapper_version()'
                             '" | idl 2> /dev/null &', shell=True,
                             stdout=sub.PIPE)
    softwares.append('plate-mapper3')
    versions.append(plate_mapper.stdout.read().decode('utf-8').strip('\n'))

    petunia = sub.Popen('readlink /home/sdss4/products/Linux64/petunia/current',
                        shell=True, stdout=sub.PIPE)
    softwares.append('Petunia')
    versions.append(petunia.stdout.read().decode('utf-8').strip('\n'))

    autoscheduler = sub.Popen('readlink /home/sdss4/products/Linux64/'
                              'autoscheduler/current', shell=True,
                              stdout=sub.PIPE)
    softwares.append('Autoscheduler')
    versions.append(autoscheduler.stdout.read().decode('utf-8').strip('\n'))

    sdss_module = sub.Popen("readlink /home/sdss4/products/Linux64/"
                            "sdss_python_module/current", shell=True,
                            stdout=sub.PIPE)
    softwares.append('SDSS Python Module')
    versions.append(sdss_module.stdout.read().decode('utf-8').strip('\n'))

    mangadrp = sub.Popen("/home/manga/products/Linux64/mangadrp/trunk/bin/"
                         "mangadrp_version", shell=True, stdout=sub.PIPE)
    softwares.append('mangadrp')
    versions.append(mangadrp.stdout.read().decode('utf-8').strip('\n'))

    sdss_obstools = sub.Popen('pip list | grep sdss-obstools', shell=True,
                              stdout=sub.PIPE).stdout.read().decode(
        'utf-8').strip('\n')
    softwares.append(sdss_obstools.split()[0])
    versions.append(sdss_obstools.split()[-1])

    print('{:-^42}'.format('Other Versions'))
    for s, v in zip(softwares, versions):
        print('{:<20}: {:<20}'.format(s, v))


if __name__ == '__main__':
    main()



#!/usr/bin/env python3
import unittest
import subprocess as sub
from astropy.time import Time
from bin import epics_fetch


class TestEPICSFetch(unittest.TestCase):
    def test_known_date(self):
        t = Time('2020-06-07T00:00', format='isot')
        data = epics_fetch.get_data('25m:mcp:cwPositions', t.datetime,
                                    (t-1).datetime)
        epics_fetch.print_data(data)

    def test_archive(self):
        """Checks to see if the directory for new data is available to this
        computer"""
        # This serves no purpose because simply importing the library is a pass
        return 

    def test_help(self):
        """"Prints the help if -h is provided"""
        sub.call('{} -h'.format(epics_fetch.__file__), shell=True)


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python
import argparse
from pathlib import Path
from astropy.time import Time
from scipy.optimize import leastsq
import fitsio
import numpy as np
import textwrap

try:
    from bin import epics_fetch
except ImportError as e:
    raise ImportError('Please add ObserverTools/bin to your PYTHONPATH:\n'
                      '    {}'.format(e))

__version__ = '3.2.1'


class APOGEERaw:
    """A class to parse raw data from APOGEE. The purpose of this class is to
    read raw image files from /data/apogee/archive, regardless of any future
    changes in SDSS. This
    will hopefully help SDSS-V logging.

    Methods:
        compute_offets: Returns the offsets of an arc/object to compute the
        relative dither

        ap_test: Returns a list of faint fibers and missing fibers from a flat
        """

    def __init__(self, fil, args, ext=1, ):
        self.file = Path(fil)
        self.ext = ext
        self.args = args
        header = fitsio.read_header(fil, ext=ext)
        self.telemetry = epics_fetch.telemetry
        dithers = self.telemetry.get('25m:apogee:ditherNamedPositions',
                                     start=(Time.now() - 5 / 24 / 60).datetime,
                                     end=Time.now().datetime,
                                     scan_archives=False, interpolation='raw')
        # layer = self.image[layer_ind]
        # An A dither is DITHPIX=12.994, a B dither is DITHPIX=13.499
        if (header['DITHPIX'] - dithers.values[-1][0]) < 0.05:
            self.dither = 'A'
        elif (header['DITHPIX'] - dithers.values[-1][1]) < 0.05:
            self.dither = 'B'
        else:
            self.dither = '{:.1f}'.format(header['DITHPIX'])
        self.exp_time = header['EXPTIME']
        self.isot = Time(header['DATE-OBS'])  # Local
        self.plate_id = header['PLATEID']
        self.cart_id = header['CARTID']
        self.exp_id = int(str(fil).split('-')[-1].split('.')[0])
        if header['EXPTYPE'].capitalize() == 'Arclamp':
            if header['LAMPUNE']:
                self.exp_type = 'UNe Arc'
            elif header['LAMPTHAR']:
                self.exp_type = 'ThAr Arc'
            else:
                print('Could not process exposure type of {}'.format(self.file))
        else:
            self.exp_type = header['EXPTYPE'].capitalize()
        self.n_read = header['NREAD']
        self.lead = header['PLATETYP']
        self.img_type = header['IMAGETYP'].capitalize()
        if header['EXPTYPE'] == 'OBJECT':
            self.seeing = header['SEEING']
        else:
            self.seeing = 0.0

        self.quickred_data = np.array([[]])
        self.quickred_file = ''
        self.utr_file = ''
        self.utr_data = np.array([[]])

    # noinspection PyTupleAssignmentBalance,PyTypeChecker
    def compute_offset(self, fibers=(30, 35), w0=939, dw=40, sigma=1.2745):
        """This is based off of apogeeThar.OneFileFitting written by Elena. It
        is supposed to generate a float for the pixel offsets of an APOGEE
        ThAr cal. Here is how it works:
        It opens a quickred file, which is of shape n_fiber*n_dispersion_pixels,
        and then it averages the fibers inside the fibers tuple. It then based
        off of the provided w0 (mean) and sigma, it creates a gaussian function
        and then creates a function called err_func that compares the gaussian
        with a slice of the data, from w0-dw/2 to w0+dw/2. It then uses scipy's
        least squared equation solver to find the difference between w0 given as
        an input and the actual w0 of the spectral line. This only works if you
        pick a prominent line to go off of. The default parameters are given for
        ThAr lines, but UNe lines could also be used, with the following inputs:
        fibers: (30, 35)
        w0: 1761
        dw: 20
        sigma: 3
        """
        w0 = int(w0)
        dw = int(dw)
        mjd = self.file.absolute().parent.name
        self.quickred_file = (self.file.absolute().parent.parent.parent
                              / 'quickred/{}/ap1D-a-{}.fits.fz'
                                ''.format(mjd, self.exp_id))
        try:
            if not self.quickred_data:
                self.quickred_data = fitsio.read(self.quickred_file, 1)
        except OSError as e:
            if self.args.verbose:
                print('Offsets for {} produced this error\n{}'.format(self.file,
                                                                      e))
            return np.nan
        lower = w0 - dw // 2
        upper = w0 + dw // 2
        line_inds = np.arange(self.quickred_data.shape[1])[lower:upper]
        line = np.average(self.quickred_data[fibers[0]:fibers[1], lower:upper],
                          axis=0)

        def fit_func(w, x):
            return np.exp(-0.5 * ((x - w) / sigma) ** 2)

        def err_func(w, x, y):
            return fit_func(w, x) - y

        w_model, success = leastsq(err_func, w0, args=(line_inds, line))

        diff = w_model[0] - w0
        if np.abs(diff) > 10:
            print('A large dither was reported for exposure {}: {:.3f}'
                  '\n  This may mean the zero point needs to be'
                  ' adjusted, currently it is {}'
                  ''.format(self.exp_id, diff, w0))
        return diff

    def ap_test(self, ws=(900, 910), master_col=None, plot=False, legacy=False,
                dome_flat_shape=None, n_fibers=300):
        if master_col is None:
            raise ValueError("APTest didn't receive a valid master_col: {}"
                             "".format(master_col))
        if legacy:
            mjd = self.file.absolute().parent.name
            self.utr_file = Path('/data/apogee/utr_cdr/{}/apRaw-{}.fits'.format(
                mjd, self.exp_id))
            if not self.utr_file.exists():
                self.utr_file = (Path.home() / 'data/apogee/utr_cdr/'
                                               '{}/apRaw-{}.fits'.format(
                    mjd, self.exp_id))
                if not self.utr_file.exists():
                    raise FileNotFoundError("Couldn't fine the file: {}".format(
                        self.utr_file.as_posix()
                    ))
            try:
                self.utr_data = fitsio.read(self.utr_file, 0)
            except OSError as e:
                if self.args.verbose:
                    print('APTest for {} produced this error\n{}'.format(
                        self.file, e))
            slc0 = np.average(self.utr_data[::-1, ws[0]:ws[1]], axis=1)
            # print(slc0.mean(), slc0.shape, slc0[0])
            slc = np.zeros(n_fibers)
            for j in range(n_fibers):
                for k in range(10):
                    if dome_flat_shape[j, k] != 0:
                        slc[j] += slc0[dome_flat_shape[j, k]]
            # print(slc.mean(), slc.shape, slc[100])

        else:
            if self.quickred_data.size == 0:
                mjd = self.file.absolute().parent.name
                self.quickred_file = (self.file.absolute().parent.parent.parent
                                      / 'quickred/{}/ap1D-a-{}.fits.fz'
                                        ''.format(mjd, self.exp_id))
                try:
                    self.quickred_data = fitsio.read(self.quickred_file, 1)
                except OSError as e:
                    if self.args.verbose:
                        print('APTest for {} produced this error\n{}'.format(
                            self.file, e))
            slc = np.average(self.quickred_data[:, ws[0]:ws[1]], axis=1)
        # print(master_col.mean(), master_col.shape, master_col[100])
        flux_ratio = slc / master_col
        # flux_ratio = flux_ratio / flux_ratio.sum() / flux_ratio.shape[0]
        flux_ratio = flux_ratio / flux_ratio.sum() * flux_ratio.shape[0]
        # print(flux_ratio.mean(), flux_ratio.shape, flux_ratio[100])
        bad_data = ((flux_ratio == np.inf)
                    | (flux_ratio == -np.inf)
                    | np.isnan(flux_ratio))
        flux_ratio[bad_data] = np.nan
        avg = np.nanmean(flux_ratio)
        missing = flux_ratio < 0.2
        faint = (flux_ratio < 0.7) & (0.2 <= flux_ratio)
        bright = ~missing & ~faint
        i_missing = np.where(missing)[0].astype(int) + 1
        i_faint = np.where(faint)[0].astype(int) + 1
        i_bright = np.where(bright)[0]
        missing_bundles = self.create_bundles(i_missing)
        faint_bundles = self.create_bundles(i_faint)
        if self.args.verbose:
            print('Missing Fibers: {}'.format(missing_bundles))
            print('Faint Fibers: {}'.format(faint_bundles))
            print()

        if plot:
            import matplotlib.pyplot as plt
            fig = plt.figure(figsize=(9, 4))
            ax = fig.gca()
            x = np.arange(len(flux_ratio)) + 1
            ax.plot(x[i_bright], flux_ratio[i_bright], 'o', c=(0, 0.6, 0.533))
            ax.plot(x[i_faint], flux_ratio[i_faint], 'o', c=(0.933, 0.466, 0.2))
            ax.plot(x[i_missing], flux_ratio[i_missing], 'o',
                    c=(0.8, 0.2, 0.066))
            ax.set_xlabel('Fiber ID')
            ax.set_ylabel('Throughput Efficiency')
            ax.axis([1, 300, -0.2, 1.35])
            ax.grid(True)
            ax.axhline(0.7, c=(0, 0.6, 0.533))
            ax.axhline(0.2, c=(0.933, 0.466, 0.2))
            ax.set_title('APOGEE Fiber Relative Intensity {}'.format(
                self.exp_id), size=15)
            fig.show()

        return missing_bundles, faint_bundles, avg

    @staticmethod
    def create_bundles(subset):
        """This method converts an array of ints into a list of strings that
        describe a large series of fibers and ints for lone fibers.

        Ex: [1, 2, 3, 5] -> ['1 - 3', 5]

        """
        if len(subset) == 0:
            return []
        bundles = [subset[0]]
        b = 0
        for fib in subset[1:]:
            if isinstance(bundles[b], np.int64):
                if bundles[b] + 1 == fib:
                    if fib % 30 == 0:
                        bundles.append(fib)
                        b += 1
                    else:
                        # All strings are created here
                        bundles[b] = '{} - {}'.format(bundles[b], fib)
                else:
                    bundles.append(fib)
                    b += 1
            elif isinstance(bundles[b], str):
                if int(bundles[b].split()[-1]) + 1 == fib:
                    if fib % 30 == 1:
                        bundles.append(fib)
                        b += 1
                    else:
                        bundles[b] = '{} - {}'.format(
                            bundles[b].split()[0], fib)
                else:
                    bundles.append(fib)
                    b += 1
        # for i, bundle in enumerate(bundles):
        #     if isinstance(bundle, str):
        #         low, high = np.array(bundle.split(' - ')).astype(int)
        #         if ((low - 1) // 30) == ((high - 1) // 30):
        #             bundles[i] = '{} bundle'.format(low)
        return bundles


def main():
    parser = argparse.ArgumentParser()
    parser.add_argument('-t', '--today', action='store_true',
                        help="Whether or not you want to search for today's"
                             " data, whether or not the night is complete."
                             " Note: must be run after 00:00Z")
    parser.add_argument('-m', '--mjd',
                        help='If not today (-t), the mjd to search')
    parser.add_argument('-v', '--verbose', action='count', default=1,
                        help='Show details, can be stacked')
    args = parser.parse_args()
    if args.today:
        mjd_today = int(Time.now().sjd)
        data_dir = '/data/apogee/archive/{}/'.format(mjd_today)
    elif args.mjd:
        data_dir = '/data/apogee/archive/{}/'.format(args.mjd)
    else:
        raise Exception('No date specified')
    # print(data_dir)
    for path in Path(data_dir).rglob('apR*.apz'):
        print(path)


if __name__ == '__main__':
    main()



#!/usr/bin/env python3
import tpmdata
import pprint
import numpy as np
import matplotlib.pyplot as plt
from matplotlib import dates
from astropy.time import Time
from argparse import ArgumentParser
from matplotlib import animation
import astropy.units as u


__version__ = '3.0.1'


tpmdata.tinit()


class StripChart:
    def __init__(self, key, fig, ax):
        self.key = key
        # self.fig = plt.figure(figsize=(6, 4))
        # self.ax = self.fig.gca()
        self.fig = fig
        self.ax = ax
        self.formatter = dates.DateFormatter('%H:%M')
        data = tpmdata.packet(1, 1)

        self.t0 = Time.now()
        self.times = Time([data['ctime']], format='unix')
        self.values = np.array([data[self.key]])

        self.line = plt.plot(self.times.plot_date, self.values)
        self.ax.xaxis.set_major_formatter(self.formatter)
        # self.fig.canvas.show()
        # plt.draw()

    def update(self, i):

        data = tpmdata.packet(1, 1)
        self.times = Time(np.append(self.times, Time(data['ctime'],
                                                     format='unix')))

        self.values = np.append(self.values, data[self.key])
        sorter = np.argsort(self.times)
        self.times = self.times[sorter]
        self.values = self.values[sorter]
        cutoff = self.times >= self.t0
        self.times = self.times[cutoff]
        self.values = self.values[cutoff]
        if self.times.shape == (0,):
            print('No times to plot')
            return
        self.ax.clear()
        # print(self.times.to_datetime(), self.values)
        self.ax.plot(self.times.to_datetime(), self.values, linewidth=3,
                     label=self.key)
        self.ax.xaxis.set_major_formatter(self.formatter)
        self.ax.axhline(self.values[0], c='r', linewidth=1,
                        label=f'Initial Value {self.key}')
        self.ax.legend()


def parseargs():
    parser = ArgumentParser(description='A tool to create strip charts using'
                                        ' the TPM')
    parser.add_argument('-c', '--channels', nargs='+', default=['dewar_sp1_lb'],
                        help='Channel(s) to query and print')
    parser.add_argument('-p', '--plot', action='store_true',
                        help='Channel(s) to plot')
    parser.add_argument('-v', '--verbose', action='store_true')
    parser.add_argument('--version', action='store_true')
    parser.add_argument('--dt', nargs=1, type=float, default=5,
                        help='Time interval between prints (used with'
                             ' --channels)')
    parser.add_argument('--list-channels', dest='list_channels',
                        action='store_true',
                        help='Prints a list of all queriable channels')

    args = parser.parse_args()

    return args


def main(args=parseargs()):

    if args.list_channels:
        args.channels = []

    if args.version:
        print(__version__)

    if args.list_channels:
        data = tpmdata.packet(1, 1)
        pprint.pprint(data.keys())

    if args.plot:
        charts = []
        anis = []
        for channel in args.channels:
            fig = plt.figure(figsize=(6, 4))
            ax = fig.add_subplot(1, 1, 1)
            chart = StripChart(channel, fig, ax)
            anis.append(animation.FuncAnimation(fig, chart.update,
                                                interval=args.dt * 1000))
            charts.append(chart)
        print('Close the plots to get a table feed')
        plt.show()

    if args.channels:
        print()
        print(f"{'Time':10}" + ''.join([' {:<12}'.format(channel)
                                        for channel in args.channels]))
        while True:
            data = tpmdata.packet(1, 1)
            old_t = Time(data['ctime'], format='unix')
            new_t = old_t
            loop_cond = True
            while loop_cond:
                data = tpmdata.packet(1, 1)
                new_t = Time(data['ctime'], format='unix')
                # print((new_t - old_t).to(u.s))
                loop_cond = (new_t - old_t) < (args.dt * u.s)
            print(f'{new_t.isot[11:19]:<10}' + ''.join([' {:12}'.format(
                data[channel]) for channel in args.channels]))


if __name__ == '__main__':
    main()



#!/usr/bin/env python3
import unittest
import pyds9
from pathlib import Path
from bin import ds9_live


class TestDS9(unittest.TestCase):
    """ These tests don't work very well because they interfere with each other,
    I'm not sure why or how to fix it
    """
    def test_pyds9(self):
        print('pyds9.test is disabled because it is quite slow')
        pyds9.test()

    def test_no_args(self):
        args = ds9_live.parseargs()
        window = ds9_live.DS9Window(args.name, args.fits_dir, args.regex,
                                    args.scale, args.zoom, args.verbose, False,
                                    True)
        window.close()

    def test_boss(self):
        if Path('/data/spectro').exists():
            window = ds9_live.DS9Window('BOSS', '/data/spectro/', 'sdR-r1*',
                                        'histequ', '1.0', False, False, True)
        else:
            window = ds9_live.DS9Window('BOSS', '~/data/spectro/', 'sdR-r1*',
                                        'histequ', '1.0', False, False, True)
        window.close()

    def test_guider(self):
        if Path('/data/gcam').exists():
            window = ds9_live.DS9Window('Guider', '/data/gcam/', 'gimg-*',
                                        'histequ', '1.0', False, False, True)
        else:
            window = ds9_live.DS9Window('Guider', '~/data/gcam/', 'gimg-*',
                                        'histequ', '1.0', False, False, True)
        window.close()

    def test_apogee_summary_dir(self):
        """Checks to see if the directory for new data is available to this
        computer"""
        args = ds9_live.parseargs()
        self.assertTrue(Path(args.fits_dir).exists())


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python3
"""This is a tool to help us parse SOS outputs during the night
Author: Dylan Gatlin
"""

import sys
import numpy as np
from bs4 import BeautifulSoup
from argparse import ArgumentParser
from pathlib import Path

try:
    import sjd
except ImportError as e:
    from bin import sjd


__version__ = '3.0.1'

sys.setrecursionlimit(10000)  # This is a very dangerous operation that
# BeautifulSoup seems to need for many SOS pages. It needs to recursively look
# through the whole html file to make a fancy python object out of it. If this
# number is too high, it could easily trigger a segmentation fault


class Plate:
    def __init__(self, plate_id, sjds, args):
        self.plate_id = plate_id
        self.sjds = sjds
        self.args = args
        self.useful_sjds = []
        self.all_exp_ids = []
        self.all_snrs = []
        self.snr_totals = []

    def parse_sjd(self, mjd):
        if self.args.verbose:
            print('Searching {}'.format(mjd))
        log_html = Path('/data/boss/sos/{}/logfile-{}.html'.format(mjd, mjd))
        if not log_html.exists():
            return
        log_soup = BeautifulSoup(log_html.open('r').read(), 'html.parser')
        exp_ids = []
        exp_snrs = []
        for plate in log_soup.find_all('caption'):
            if self.plate_id in plate.find('b').decode():
                for line in plate.find('tr').decode().split('<tr>'):
                    if '>(S/N)^2' in line:
                        sci_images = line.split('<td align="RIGHT">')
                        exp_ids.append(sci_images[0].split('SCIENCE-')[-1][:-1])
                        if 'bold' not in sci_images[2]:
                            exp_snrs.append(sci_images[2:4])
                        else:
                            exp_snrs.append([sci_images[2].split()[-1],
                                             sci_images[3].split()[-1]])
                last_line = plate.find_all('tr')[-1].decode().split(
                    '<td align="RIGHT">')
                try:
                    last_line[1]
                except IndexError:
                    continue
                if 'TOTAL (S/N)^2' in last_line[1]:
                    self.useful_sjds.append(mjd)
                    self.all_exp_ids.append(np.array(exp_ids).astype(int))
                    self.all_snrs.append(np.array(exp_snrs).astype(float))
                    # Sometimes there's a bold secion in the S/N values, this
                    # loop ignores any of those values on the left
                    # The if is in case there is no reported S/N^2 (I think tha
                    # means the exposures were bad or no science exposures were
                    # taken.
                    if (('bold' not in last_line[2])
                       and ('bold' not in last_line[3])):
                        self.snr_totals.append(last_line[2:4])
                    else:
                        if 'bold' not in last_line[2].split()[-1]:
                            self.snr_totals.append([last_line[2].split()[-1],
                                                    last_line[3].split()[-1]])
                        else:
                            self.snr_totals.append([0., 0.])

    def parse_plate(self):
        for mjd in self.sjds:
            self.parse_sjd(mjd)
        # print(self.useful_sjds)
        self.useful_sjds = np.array(self.useful_sjds).astype(int)
        self.all_exp_ids = self.all_exp_ids
        self.all_snrs = np.array(self.all_snrs)
        self.snr_totals = np.array(self.snr_totals).astype(float)

    def print_summary(self):

        for i, mjd in enumerate(self.useful_sjds):
            print('    SJD: {}'.format(mjd))
            print('        {:^8} {:^4} {:^4}'.format('ExpID', 'R1', 'B1'))
            for expid, snr in zip(self.all_exp_ids[i], self.all_snrs[i]):
                print('        {:0>8.0f} {:4.1f} {:4.1f}'.format(
                    expid, snr[0], snr[1]))
            print('    Night total: {:4.1f} {:4.1f}'.format(
                *self.snr_totals[i]))
        if bool(self.snr_totals.sum()):
            print('\nTotal S/N^2: {:4.1f} {:4.1f}\n'.format(
                    *self.snr_totals.sum(axis=0)))

        else:
            print("This plate hasn't been observed\n")


def parse_args():
    parser = ArgumentParser(description='A tool to help guide observations by'
                                        ' reading SOS S/N^2 and summing the'
                                        ' S/N^2 across multiple days. Default'
                                        ' usage is to use the -p argument for'
                                        ' each plate you want, which will check'
                                        ' the last 40 mjds.')
    parser.add_argument('-p', '--plates', nargs='+',
                        help='Plate numbers to create a summary for')
    parser.add_argument('-m', '--mjds', dest='sjds', nargs='+',
                        help='mjds (actually sjds) to search')
    parser.add_argument('-v', '--verbose', action='store_true',
                        help='Verbosity for debugging')
    parser.add_argument('--version', action='store_true',
                        help='Print script version')
    args = parser.parse_args()

    if args.version:
        print(__version__)

    if not args.sjds:
        today = sjd.sjd()
        first_sjd = today - 40
        args.sjds = np.arange(first_sjd, today + 1).astype(int)

    return args


def main():
    args = parse_args()

    for plate in args.plates:
        print('Plate: {}'.format(plate))
        pl = Plate(plate, args.sjds, args)
        pl.parse_plate()
        pl.print_summary()


if __name__ == '__main__':
    main()



#!/usr/bin/env python3
"""
help.py

This is a file to pull the help for all programs in bin. Each script must be
 added individually, some scripts use argparse, so -h is called, others do not,
 and so their __doc__ is printed. Based on spHelp written by Elena.

2020-06-08  DG  Init, based on spHelp
2020-08-17  DG  Added XMID and WAVEMID
2020-10-18  DG  Added m4l
"""
import subprocess as sub
from pathlib import Path
from bin import sjd, sp_version, m4l

__version__ = '3.1.0'

argparsed = ['ap_test.py', 'boss_sha1sum.py', 'ds9_live.py', 'epics_fetch.py',
             'get_dust.py', 'sloan_log.py', 'tpm_fetch.py', 'tpm_feed.py']
non_argparsed = ['sjd.py', 'sp_version.py', 'wave_mid.py', 'x_mid']
bin_dir = Path(__file__).parent

for arg in argparsed:
    print('{:=^80}'.format(arg))
    sub.call([bin_dir / arg, '-h'])
    print()

# These scripts don't use argparse, but they do have a docstring and a main
# function, so they can be safely imported and we'll just print __doc__
print('{:=^80}'.format('sjd.py'))
print(sjd.__doc__)
print('{:=^80}'.format('sp_version.py'))
print(sp_version.__doc__)
print('{:=^80}'.format('m4l.py'))
print(m4l.__doc__)

# Because these files don't have a main syntax, we can't import them and print
# __doc__, so we have to read them manually
wavemid = open(bin_dir / 'wave_mid.py', 'r').read()
print('{:=^80}'.format('wave_mid.py'))
print(wavemid.split('"""')[1])

xmid = open(bin_dir / 'x_mid.py', 'r').read()
print('{:=^80}'.format('x_mid.py'))
print(xmid.split('"""')[1])



#!/usr/bin/env python3
"""
Description:	Opens and continuously updates ds9 with the latest file. Run
ds9_live.py -h for details.

Changelog:
2011-06-11  JB 	Apache Point Observatory Created file from
 spds9.py
2020-05-31  DG  Replaced almost every library for a python 3 upgrade. ds9 ->
 pyds9, os.path -> pathlib, optionparse -> argparse. Updated syntax to Python
 3 and greatly improved PEP-8 compliance. Changed source of apogee data to work
 on any system
"""

import hashlib
import pyds9
import time
import fitsio
# from astropy.time import Time, TimeDelta
# import os
import numpy as np
# import tracemalloc
from argparse import ArgumentParser
from pathlib import Path

default_dir = Path('/data/apogee/utr_cdr/')
if not default_dir.exists():
    default_dir = Path.home() / 'data/apogee/utr_cdr/'
boss_cams = ['r1', 'b1', 'r2', 'b2']
file_sizes = {'APOGEE': 67115520, 'BOSS': 9e6, 'Guider': 4e5,
              'Engineering': 9e5}

__version__ = '3.1.1'


class DS9Window:
    """Displays the last image in a given directory in ds9
    inputs:
    name: The name of the window, it will add the fits dir and a hash onto this
        in the actual window
    fits_dir: a directory of dated folders where images are stored
    regex: The formatting of the fits images we want to display
    scale: The image scale
    zoom: The image zoom
    verbose: A debugging tool
    """

    def __init__(self, name, fits_dir, regex, scale, zoom, verbose, info,
                 vertical):

        # Constants and variables

        self.last_file = ''

        # Arguments
        self.name = name
        self.fits_dir = Path(fits_dir)
        self.regex = regex
        self.scale = scale
        self.zoom = zoom
        self.verbose = verbose
        self.info = info
        self.vertical = vertical

        hsh = hashlib.sha1(''.join(self.name + str(self.fits_dir) + self.regex
                                   + self.scale).encode())
        self.ds9_target = '{}:{}'.format(self.name, hsh.hexdigest()[:6])
        # self.ds9_target = 'ds9_live:{}'.format(os.getpid())

        if self.verbose:
            print('dir = {}\nName = {}\nscale = {}\nzoom = {}'.format(
                self.fits_dir, self.name, self.scale,
                self.zoom))

        # Initialize
        targets = pyds9.ds9_targets()
        if not targets:
            targets = []
        for ds9 in targets:
            if self.name in ds9:
                print('A similar instance of ds9 is already running as {},'
                      ' would you like to connect to it, close it, or create'
                      ' another window with a new name?'.format(ds9))
                action = input('[change]/close/connect: ')
                if ((action.lower() == 'connect')
                        or (action.lower() == '')):
                    self.name = ds9
                elif action.lower() == 'close':
                    d = pyds9.DS9(ds9)
                    d.set('exit')
                else:
                    print('Please provide a new name. This can also be done'
                          ' via the -n argument:')
                    self.name = input('>')
        if self.verbose:
            print(self.name)
        self.ds9 = pyds9.DS9(self.name)
        if '/data/spectro' in self.fits_dir.as_posix():
            self.ds9.set('tile yes')

        if not self.info:
            self.ds9.set('view info no')
            self.ds9.set('view panner no')
            self.ds9.set('view magnifier no')
            self.ds9.set('view buttons no')

        if self.vertical:
            self.ds9.set('view layout vertical')
            self.ds9.set('view buttons no')

    @staticmethod  # This means it doesn't take self as an argument
    def is_fits(filename):
        """Returns whether a file is a FITS file based on its extension"""
        if not isinstance(filename, Path):
            filename = Path(filename)

        if 'fit' in filename.name:
            return True
        else:
            return False

    def latest_fits_dir(self):
        """Returns the latest sub-directory"""

        max_time = -1
        dirname = ''

        # Obtain the files in the directory and add the full path to them

        for fil in Path(self.fits_dir).glob('*'):
            fil = fil.absolute()

            if fil.is_dir():

                # Store the name and mtime of only the latest FITS file, reads
                # through every file, checks its mtime, and keeps the most
                # recent for the return

                mtime = fil.stat().st_mtime
                # print max_time, file, mtime
                if max_time < mtime:
                    dirname = fil
                    max_time = mtime

        return dirname

    def latest_fits_file(self, pattern):
        """Returns the latest FITS file matching <pattern>"""

        # Obtain the files in the directory and add the full path to them

        fits_dir = self.latest_fits_dir()

        # print 'dir = ', dir
        img_times = []
        imgs = []
        for fil in Path(fits_dir).glob(pattern):
            fil = fil.absolute()

            # See if the file name matches the pattern and the file is a FITS
            # file

            if self.is_fits(fil):
                # Store the name and mtime of only the latest FITS file

                mtime = Path(fil).stat().st_mtime
                img_times.append(mtime)
                imgs.append(fil)
                # print max_time, file, mtime
                # if max_time < mtime:
                # fits_filename = fil
                # max_time = mtime
        img_times = np.array(img_times)
        imgs = np.array(imgs)
        sorter = img_times.argsort()
        imgs = imgs[sorter]
        # img_times = img_times[sorter]
        # An attempt at making sure that if APOGEE isn't on the summary
        # directory, it won't crash because it won't try to read an image
        # that is still writing
        try:
            if (('APOGEE' in self.name)
                    and ('summary' not in self.fits_dir.as_posix())):
                fits_filename = imgs[-2].absolute()
            else:
                fits_filename = imgs[-1].absolute()
        except IndexError:
            return None

        return fits_filename

    # return sorted (fits_files.items(), key=lambda (k,v): (v,k),
    # reverse=True)[0][0]

    def display(self, fil, frame):
        """Display <file> in <frame> with optional scaling and zoom"""

        if frame >= 0 and fil.exists():
            self.ds9.set('frame {}'.format(frame))
            self.ds9.set('file {}'.format(fil))

            if self.zoom:
                self.ds9.set('zoom to {}'.format(self.zoom))

            if self.scale:
                self.ds9.set('scale {}'.format(self.scale))
        else:
            return

    def update(self):
        """Update the display"""

        fil = self.latest_fits_file(self.regex)
        if fil is None:
            print("No files found in today's folder, skipping.")
            return
        if self.verbose:
            print('Latest fits file ={}\nLast fits file   ={}'
                  ''.format(fil, self.last_file))
        if fil != self.last_file:
            if self.verbose:
                print('Displaying {}'.format(fil))
            try:
                # In case the file is incomplete, it won't crash ds9. This
                # can happen if it is either size 0 (just made), or it is made,
                # but it hasn't been populated by the first exposure.
                stats = fil.lstat()
                try:  # Handles the exists but unwritten issue
                    fitsio.read(fil)
                except OSError:
                    print('File is actively being written, skipping.')
                    return

                # Handles the size issue
                if stats.st_size < file_sizes[self.name]:
                    print('File too small, skipping. This usually happens when'
                          ' the previous image was a hartmann')
                    return
            except KeyError as e:
                if self.verbose:
                    print('Handled error: {}'.format(e))
            # Because BOSS has 4 cameras, it must loop 4 times

            if '/data/spectro' in self.fits_dir.as_posix():
                for i, cam in enumerate(boss_cams):
                    if self.verbose:
                        print(i, fil.name.replace('r1', cam), type(fil))
                    self.display(fil.parent / fil.name.replace('r1', cam), i)
            else:
                self.display(fil, 0)
            self.last_file = fil

    def close(self):
        self.ds9.set('exit')


# If run as a program, start here

def parseargs():
    # Define command line options

    parser = ArgumentParser(description='A tool to leave running continuously'
                                        ' that will display the most current'
                                        ' exposure. By default, it will'
                                        ' run every 60 seconds. Can be run for'
                                        ' any camera. Previously called ads9,'
                                        ' spds9, gds9, etc.')
    parser.add_argument('-a', '--apogee', action='store_true',
                        help='If included, will display APOGEE images.'
                             ' Overrides most arguments')
    parser.add_argument('-b', '--boss', action='store_true',
                        help='If included, will display BOSS images.'
                             ' Overrides most arguments')
    parser.add_argument('-d', '--directory', dest='fits_dir',
                        default=default_dir, type=str,
                        help='Set FITS data directory. It needs to be a'
                             ' directory of dated folders, where the newest'
                             ' folder has the newest data.'
                             ' Default is {}'.format(default_dir))

    parser.add_argument('-e', '--ecam', action='store_true',
                        help='If included, will display engineering camera'
                             'images. Overrides most arguments.')
    parser.add_argument('-g', '--guider', action='store_true',
                        help='If included, will display guider images.'
                             ' Overrides most arguments.')
    parser.add_argument('-j', '--info', dest='info', action='store_true',
                        help='If included, it will show the info panel like a' 
                             ' normal DS9 window. Without info, it will be more' 
                             ' compact and may easily fit on the monitor')
    parser.add_argument('-i', '--interval', dest='interval', default=60,
                        type=int, help='Set the refresh rate.	Default is 5'
                                       'seconds. Refreshes will be this '
                                       'number  of seconds apart.')
    parser.add_argument('-n', '--name', dest='name', default='Scanner',
                        type=str,
                        help='Set ds9 Window name. Default is autogenerated.')
    parser.add_argument('-r', '--regex', default='apRaw*',
                        help='A regex to match the fits files, default is'
                             ' {}'.format('apRaw*'))
    parser.add_argument('-s', '--scale', dest='scale', default='histequ',
                        type=str, help='Set scaling. Default is "histequ"')

    parser.add_argument('--version', action='store_true', help='Version info')
    parser.add_argument('-u', '--vertical', dest='vertical',
                        action='store_true', help='Whether or not to have an '
                                                  'information screen on the '
                                                  'side')
    parser.add_argument('-v', '--verbose', action='store_true', dest='verbose',
                        default=False,
                        help='Be verbose. Default is to be quiet.')
    parser.add_argument('-z', '--zoom', dest='zoom', default='1.0',
                        type=str, help='Set zoom factor. Default is 1.0')

    args = parser.parse_args()

    if args.verbose:
        print('interval = {}'.format(args.interval))
        print('scale = {}'.format(args.scale))
        print('zoom = {}'.format(args.zoom))

    if args.version:
        print(__version__)

    if args.apogee and args.boss:
        raise Exception('Cannot do both boss and apogee in one script, sorry')

    if args.apogee:
        if Path('/summary-ics/').exists():
            args.fits_dir = Path('/summary-ics')
        else:
            args.fits_dir = Path('/data/apogee/utr_cdr/')
        if args.name == 'Scanner':
            args.name = 'APOGEE'
        args.scale = args.scale
        args.zoom = args.zoom

    elif args.boss:
        args.fits_dir = Path('/data/spectro/')
        if args.name == 'Scanner':
            args.name = 'BOSS'
        args.scale = args.scale
        args.zoom = 0.5
        args.regex = 'sdR-r1*'

    elif args.guider:
        args.fits_dir = Path('/data/gcam/')
        args.name = 'Guider'
        args.scale = args.scale
        args.zoom = args.zoom
        args.regex = 'gimg-*'

    elif args.ecam:
        args.fits_dir = Path('/data/gcam/')
        args.name = 'Engineering'
        args.scale = args.scale
        args.zoom = args.zoom
        args.regex = 'proc-gimg-*'

    return args


def main():
    # tracemalloc.start()
    args = parseargs()
    # Start the display
    window = DS9Window(args.name, args.fits_dir, args.regex, args.scale,
                       args.zoom, args.verbose, args.info, args.vertical)
    while True:
        window.update()
        # This loop is for tracing memory allocation to track memory leaks, none
        # were found, so this count is hidden to avoid cluttering verbose
        if args.verbose:
            pass
            # snapshot = tracemalloc.take_snapshot()
            # top_stats = snapshot.statistics('lineno')
            # for stat in top_stats[:5]:
            # print(stat)

        time.sleep(args.interval)


if __name__ == '__main__':
    main()



#!/usr/bin/env python3
"""
sjd.py
Written by Elena Malanushenko, moved to ObserverTools by Dylan Gatlin

If run as a script, it prints the Sloan Julian Date, which is mjd + 0.3.
 This tool is often imported in other
 Python functions. This is probably the most used script in this library, so
 stability is critical.

Changelog:

2020-07-02  DG  Renamed sjd inside sjd function to current_sjd
"""
import time

__version__ = '3.2.1'

TAI_UTC = 0  # TAI_UTC =34;
aSjd = 40587.25
bSjd = 86400.0


def sjd():
    # current mjd
    d = time.gmtime()
    tt = time.mktime(d)
    current_sjd = (tt + TAI_UTC) / bSjd + aSjd
    return int(current_sjd)


def sjd_to_time(sjd_float):
    sjd1 = sjd_float
    tm = (sjd1 - aSjd) * bSjd - TAI_UTC
    return tm  # time in seconds time.time()


def main():
    print(sjd())


if __name__ == "__main__":
    main()



#!/usr/bin/env python3
import unittest
import sp_version
import subprocess as sub


class TestSPVersion(unittest.TestCase):
    def test_no_args(self):
        output = sub.Popen(sp_version.__file__, stderr=sub.PIPE)
        for line in output.stderr.read().decode('utf-8').splitlines():
            print(line)
            self.assertNotIn('not found', line,
                             msg='Package manager or package not found')


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python3
import unittest
import signal
from pathlib import Path
import tpm_feed


class TestTPMFeed(unittest.TestCase):
    def setUp(self):
        self.file = Path(__file__).absolute().parent.parent / 'bin/tpm_feed.py'

        class DummyArgs(object):
            pass

        self.args = DummyArgs()
        self.args.channels = ['dewar_sp1_lb']
        self.args.plot = True
        self.args.verbose = True
        self.args.version = False
        self.args.dt = 5
        self.args.list_channels = False

    @staticmethod
    def handler(signum, frame):
        print('Exiting call')
        raise TimeoutError('The function reached timeout without other errors')

    def test_print(self):
        signal.signal(signal.SIGALRM, self.handler)
        signal.alarm(10)
        try:
            tpm_feed.main(args=self.args)
        except TimeoutError as t:
            print(t)

    def test_list_channels(self):
        self.args.plot = False
        self.args.list_channels = True
        tpm_feed.main(args=self.args)


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python3
import unittest
from pathlib import Path
import tpm_fetch


class TestTPMFetch(unittest.TestCase):
    def setUp(self):
        self.file = Path(__file__).absolute().parent.parent / 'bin/tpm_fetch.py'

        class DummyArgs(object):
            pass

        self.args = DummyArgs()
        self.args.t1 = '2020-11-05T06:00:00'
        self.args.t2 = '2020-11-06T06:30:00'
        self.args.mjd = None
        self.args.channels = ['alt_pos', 'az_pos']
        self.args.verbose = True

    def test_time_range(self):
        tpm_fetch.main(args=self.args)

    def test_one_mjd(self):
        self.args.t1 = None
        self.args.t2 = None
        self.args.mjd = 59159
        tpm_fetch.main(args=self.args)


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python3
import unittest
import subprocess as sub
from pathlib import Path


class TestWAVEMID(unittest.TestCase):

    def test_4cam(self):
        """Runs a test from a normal 4-camera xmid
        """
        here = Path(__file__).absolute().parent
        bin_dir = here.parent / 'bin'
        x = sub.check_output(' '.join([(bin_dir / 'WAVEMID').as_posix(),
                                       '4907.9', '7989.9', '5000.8', '8037.1']),
                             shell=True)
        self.assertNotIn(x.decode('utf-8'), 'Traceback')

    def test_2cam(self):
        """Runs a test from a normal 4-camera xmid
        """
        here = Path(__file__).parent.absolute()
        bin_dir = here.parent / 'bin'
        x = sub.check_output([(bin_dir / 'WAVEMID').as_posix(), '4907.9',
                              '7989.9'])
        self.assertNotIn(x.decode('utf-8'), 'Traceback')


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python3
import unittest
import subprocess as sub
from pathlib import Path


class TestXMID(unittest.TestCase):

    def test_4cam(self):
        """Runs a test from a normal 4-camera xmid
        """
        here = Path(__file__).parent.absolute()
        bin_dir = here.parent / 'bin'
        x = sub.check_output([bin_dir / 'XMID', '2043.7', '2046.8', '2042.7',
                              '2019.0'])
        self.assertNotIn(x.decode('utf-8'), 'Traceback')

    def test_2cam(self):
        """Runs a test from a normal 4-camera xmid
        """
        here = Path(__file__).parent.absolute()
        bin_dir = here.parent / 'bin'
        x = sub.check_output([bin_dir / 'XMID', '2049.2', '2046.9'])
        self.assertNotIn(x.decode('utf-8'), 'Traceback')


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python3
import unittest
import subprocess as sub
from bin import get_dust


class TestMJD(unittest.TestCase):
    def test_main(self):
        sub.call(get_dust.__file__, shell=True)


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python3

"""A logging tool that is meant to perform the function of LogSupport scripts in
STUI, but by bypassing STUI and directly accessing telemetry

2020-01-28  dgatlin     Init, after just learning about telemetry queries
2020-06-20  dgatlin     Changed some times around, it's very easy to mix up sjd
    and mjd in channelarchiver
"""
from astropy.time import Time
import argparse

try:
    from bin import epics_fetch
except ImportError as e:
    raise ImportError('Please add ObserverTools/bin to your PYTHONPATH:\n'
                      '    {}'.format(e))

__version__ = '3.2.0'


class LogSupport:
    def __init__(self, tstart, tend, args):
        self.tstart = Time(tstart)
        self.tend = Time(tend)
        self.args = args
        self.telemetry = epics_fetch.telemetry

        self.call_times = []
        self.offsets = ''
        self.focus = ''
        self.weather = ''
        self.hartmann = ''

    def query(self, key, tstart, tend, tel_dict):
        data = self.telemetry.get(key, tstart, tend,
                                  interpolation='raw', scan_archives=False)
        if isinstance(key, list):
            for k, d in zip(key, data):
                tel_dict[k].append(d.values[0])
        else:
            value = data.values[0]
            tel_dict[key].append(value)
        return data

    def set_callbacks(self):
        data = self.telemetry.get([
            # '25m:sop:doApogeeBossScience_nDither'
            # ':nDitherDone',
            # '25m:sop:doBossScience_nExp'
            # ':nExpDone',
            # '25m:sop:doMangaSequence_ditherSeq:index',
            # '25m:sop:doApogeeMangaSequence_ditherSeq:'
            # 'index',
            # '25m:sop:doApogeeScience_index:index'],
            '25m:apogee:exposureWroteSummary'],
            (self.tstart - 0.3).datetime,
            (self.tend - 0.3).datetime,
            interpolation='raw',
            scan_archives=False)
        self.call_times = []
        for dat in data:
            self.call_times += dat.times
        self.call_times = Time(self.call_times)
        callback_sorter = self.call_times.argsort()
        self.call_times = self.call_times[callback_sorter]

        if self.args.verbose:
            print('Callback start: {}'.format(self.tstart.isot))
            print('Callback end: {}'.format(self.tend.isot))
            print(self.call_times)

        filt = self.tstart - 0.3 < self.call_times
        self.call_times = self.call_times[filt]

    def get_offsets(self):
        offsets_keys = ['25m:guider:cartridgeLoaded:cartridgeID',
                        '25m:guider:cartridgeLoaded:plateID',
                        '25m:guider:survey:plateType',
                        '25m:tcc:axePos:az', '25m:tcc:axePos:alt',
                        '25m:tcc:axePos:rot', '25m:tcc:objArcOff:az',
                        '25m:tcc:objArcOff:alt', '25m:tcc:guideOff:rot',
                        '25m:tcc:calibOff:az', '25m:tcc:calibOff:alt',
                        '25m:tcc:calibOff:rot', '25m:guider:guideRMS:RMSerror']
        off_data = {}
        for key in offsets_keys:
            off_data[key] = []
        for time in self.call_times:
            self.query(offsets_keys, time.datetime, time.datetime,
                       off_data)

        self.offsets += '=' * 80 + '\n'
        self.offsets += '{:^80}\n'.format(
            'Telescope Offsets and Scale (arcsec)')
        self.offsets += '=' * 80 + '\n\n'
        self.offsets += ('{:<5} {:<9} {:<6} {:<4} {:<6} {:<13} {:<8} {:<10}'
                         ' {:<8}\n'.format('Time', 'Cart', 'Az', 'Alt', 'Rot',
                                           '(\u03B4RA, \u03B4Dec)', 'guideRot',
                                           'calibOff', 'guideRMS'))
        self.offsets += '-' * 80 + '\n'
        for i, time in enumerate(self.call_times):
            if off_data[offsets_keys[2]][i] == '':
                continue
            self.offsets += ('{:>5} {:>2}-{:0>5} {:>1} {:>6.1f} {:>4.1f}'
                             ' {:>6.1f} ({:>5.1f},{:>5.1f}) {:>8.1f}'
                             ' ({:2.0f},{:2.0f},{:2.0f}) {:>8.3f}'
                             '\n'.format(time.isot[11:16],
                                         off_data[offsets_keys[0]][i],
                                         off_data[offsets_keys[1]][i],
                                         off_data[offsets_keys[2]][i][0],
                                         off_data[offsets_keys[3]][i],
                                         off_data[offsets_keys[4]][i],
                                         off_data[offsets_keys[5]][i],
                                         off_data[offsets_keys[6]][i][0] * 3600,
                                         off_data[offsets_keys[7]][i][0] * 3600,
                                         off_data[offsets_keys[8]][i][0] * 3600,
                                         off_data[offsets_keys[9]][i][0] * 3600,
                                         off_data[offsets_keys[10]][
                                             i][0] * 3600,
                                         off_data[offsets_keys[11]][
                                             i][0] * 3600,
                                         off_data[offsets_keys[12]][i])
                             )

    def get_focus(self):
        focus_keys = ['25m:guider:cartridgeLoaded:cartridgeID',
                      '25m:guider:cartridgeLoaded:plateID',
                      '25m:guider:survey:plateType',
                      '25m:tcc:scaleFac', '25m:tcc:primOrient:pos',
                      '25m:tcc:secOrient:piston', '25m:tcc:secFocus',
                      '25m:tcc:axePos:az', '25m:tcc:axePos:alt',
                      '25m:apo:airTempPT', '25m:apo:winds',
                      '25m:apo:windd', '25m:guider:seeing']
        foc_data = {}
        for key in focus_keys:
            foc_data[key] = []
        for time in self.call_times:
            self.query(focus_keys, time.datetime, time.datetime,
                       foc_data)

        self.focus += '=' * 80 + '\n'
        self.focus += '{:^80}\n'.format('Telescope Focus')
        self.focus += '=' * 80 + '\n\n'
        self.focus += ('{:<5} {:<9} {:<6} {:<5} {:<5} {:<5} {:<6} {:<5}'
                       ' {:<5} {:<4} {:<3}'
                       ' {:<4}\n'.format('Time', 'Cart', 'Scale', 'M1', 'M2',
                                         'Focus', 'Az', 'Alt', 'Temp',
                                         'Wind', 'Dir', 'FWHM'))
        self.focus += '-' * 80 + '\n'
        for i, time in enumerate(self.call_times):
            if foc_data[focus_keys[2]][i] == '':
                continue
            self.focus += ('{:>5} {:>2}-{:0>5}{:>1} {:>6.1f} {:>5.0f}'
                           ' {:>5.0f}'
                           ' {:>5.0f} {:>6.1f} {:>5.1f} {:>5.1f} {:>4.0f}'
                           ' {:>3.0f}'
                           ' {:>4.1f}\n'.format(time.isot[11:16],
                                                foc_data[focus_keys[0]][i],
                                                foc_data[focus_keys[1]][i],
                                                foc_data[focus_keys[2]][i][0],
                                                (foc_data[focus_keys[3]][i]
                                                 - 1) * 1e6,
                                                foc_data[focus_keys[4]][i],
                                                foc_data[focus_keys[5]][i],
                                                foc_data[focus_keys[6]][i],
                                                foc_data[focus_keys[7]][i],
                                                foc_data[focus_keys[8]][i],
                                                foc_data[focus_keys[9]][i],
                                                foc_data[focus_keys[10]][i],
                                                foc_data[focus_keys[11]][i],
                                                foc_data[focus_keys[12]][i]))

    def get_weather(self):
        weather_keys = ['25m:guider:cartridgeLoaded:cartridgeID',
                        '25m:guider:cartridgeLoaded:plateID',
                        '25m:guider:survey:plateType',
                        '25m:apo:airTempPT', '25m:apo:dpTempPT',
                        '25m:apo:humidPT', '25m:apo:winds',
                        '25m:apo:windd', '25m:apo:dustb',
                        '25m:apo:irscsd', '25m:apo:irscmean']
        weather_data = {}
        for key in weather_keys:
            weather_data[key] = []
        for time in self.call_times:
            self.query(weather_keys, time.datetime, time.datetime,
                       weather_data)

        self.weather = '=' * 80 + '\n'
        self.weather += '{:^80}\n'.format('Weather Log')
        self.weather += '=' * 80 + '\n\n'
        self.weather += ('{:<5} {:<9} {:<5} {:<5} {:<4} {:<5} {:<4} {:<3}'
                         ' {:<6} {:<7} {:<5}'
                         '\n'.format('Time', 'Cart', 'Temp', 'DP', 'Diff',
                                     'Humid', 'Wind', 'Dir', '1\u03BCmDst',
                                     'IRSC\u03C3', 'IRSC\u03BC'))
        self.weather += '-' * 80 + '\n'
        for i, time in enumerate(self.call_times):
            if weather_data[weather_keys[2]][i] == '':
                continue
            self.weather += ('{:>5} {:>2}-{:0>5}{:>1} {:>5.1f} {:>5.1f}'
                             ' {:>4.1f} {:>5}'
                             ' {:>4.0f} {:>3.0f} {:>6.0f} {:>7.1f} {:>5.1f}'
                             '\n'.format(time.isot[11:16],
                                         weather_data[weather_keys[0]][i],
                                         weather_data[weather_keys[1]][i],
                                         weather_data[weather_keys[2]][i][0],
                                         weather_data[weather_keys[3]][i],
                                         weather_data[weather_keys[4]][i],
                                         weather_data[weather_keys[3]][i]
                                         - weather_data[weather_keys[4]][i],
                                         '{:>.0f}%'.format(
                                             weather_data[weather_keys[5]][i]),
                                         weather_data[weather_keys[6]][i],
                                         weather_data[weather_keys[7]][i],
                                         weather_data[weather_keys[8]][i],
                                         weather_data[weather_keys[9]][i],
                                         weather_data[weather_keys[10]][i]))

    def get_hartmann(self):
        data = self.telemetry.get('25m:hartmann:sp1Residuals:deg',
                                  (self.tstart - 0.3).datetime,
                                  (self.tend - 0.3).datetime,
                                  interpolation='raw',
                                  scan_archives=False)
        hart_times = data.times
        hart_times = Time(hart_times)
        hart_sorter = hart_times.argsort()
        hart_times = hart_times[hart_sorter]
        filt = self.tstart - 0.3 < hart_times
        hart_times = hart_times[filt]

        if self.args.verbose:
            print('hart_times: ', hart_times)
        hartmann_keys = ['25m:guider:cartridgeLoaded:cartridgeID',
                         '25m:guider:cartridgeLoaded:plateID',
                         '25m:guider:survey:plateType',
                         '25m:hartmann:r1PistonMove', '25m:hartmann:b1RingMove',
                         '25m:hartmann:sp1AverageMove',
                         '25m:hartmann:sp1Residuals:deg',
                         '25m:boss:sp1Temp:median']
        # '25m:hartmann:r2PistonMove', '25m:hartmann:b2RingMove
        # '25m:hartmann:sp2AverageMove',
        # '25m:hartmann:sp2Residuals:deg',
        # '25m:boss:sp2Temp:median']
        hart_data = {}
        for key in hartmann_keys:
            hart_data[key] = []
        for time in hart_times:
            self.query(hartmann_keys, time.datetime,
                       time.datetime,
                       hart_data)

        self.hartmann += '=' * 80 + '\n'
        self.hartmann += '{:^80}\n'.format('Hartmann Log')
        self.hartmann += '=' * 80 + '\n\n'
        self.hartmann += ('{:<5} {:<9} {:<5} {:<5} {:<5} {:<7} {:<4}'
                          '\n'.format('Time', 'Cart', 'R1', 'B1', 'Move1',
                                      'B1Resid', 'TSP1'))
        self.hartmann += '-' * 80 + '\n'
        for i, time in enumerate(hart_times):
            if hart_data[hartmann_keys[2]][i] == '':
                continue
            self.hartmann += ('{:>5} {:>2}-{:0>5}{:<1} {:>5.0f} {:>5.1f}'
                              ' {:>5.0f} {:>7.1f} {:>4.1f}'
                              '\n'.format(time.isot[11:16],
                                          hart_data[hartmann_keys[0]][i],
                                          hart_data[hartmann_keys[1]][i],
                                          hart_data[hartmann_keys[2]][i][0],
                                          hart_data[hartmann_keys[3]][i],
                                          hart_data[hartmann_keys[4]][i],
                                          hart_data[hartmann_keys[5]][i],
                                          hart_data[hartmann_keys[6]][i],
                                          hart_data[hartmann_keys[7]][i], ))
            # hart_data[hartmann_keys[8]][i],
            # hart_data[hartmann_keys[9]][i],
            # hart_data[hartmann_keys[10]][i],
            # hart_data[hartmann_keys[11]][i],
            # hart_data[hartmann_keys[12]][i]))


def main():
    parser = argparse.ArgumentParser()
    parser.add_argument('-t', '--today', action='store_true',
                        help="Whether or not you want to search for today's"
                             " data, whether or not the night is complete."
                             " Note: must be run after 00:00Z")
    parser.add_argument('-m', '--mjd',
                        help='If not today (-t), the mjd to search')
    parser.add_argument('-p', '--print', action='store_true',
                        help='Print all possible outputs')
    parser.add_argument('-k', '--key',  # nargs=1,
                        help='The key to be queries, space separated.'
                             '\nEx: 25m:apogee:ditherPosition:pixels')
    parser.add_argument('-o', '--offsets', action='store_true',
                        help='Whether or not to print the offsets log')
    parser.add_argument('-f', '--focus', action='store_true',
                        help='Whether or not to print the focus log')
    parser.add_argument('-w', '--weather', action='store_true',
                        help='Whether or not to print the weather log')
    parser.add_argument('--hartmann', action='store_true',
                        help='Whether or not to print the hartmann log')
    parser.add_argument('-v', '--verbose', action='store_true',
                        help='Verbose outputs for debugging')
    args = parser.parse_args()

    if args.today:
        now = Time.now() + 0.3
        sjd = int(now.mjd)
        start = Time(sjd, format='mjd')
        end = now
    elif args.mjd:
        sjd = args.mjd
        start = Time(sjd, format='mjd')
        end = Time(int(sjd) + 1, format='mjd')
    else:
        raise argparse.ArgumentError(args.mjd,
                                     'Must provide -t or -m in arguments')

    if args.print:
        args.offsets = True
        args.focus = True
        args.weather = True
        args.hartmann = True

    if args.verbose:
        print('Start: {}'.format(start))
        print('End: {}'.format(end))

    if args.key:
        tel = LogSupport(start, end, args)
        tel.set_callbacks()
        dummy_dic = {args.key: []}
        data = []
        print('Key: {}'.format(args.key))
        print('Time: Value')
        for time in tel.call_times:
            data = tel.query(args.key, time.datetime, time.datetime, dummy_dic)
            print('{}: {}'.format(data.times[0], data.values[0]))

        print('Units: {}'.format(data.units))

    tel = LogSupport(start, end, args)
    tel.set_callbacks()

    if args.offsets:
        tel.get_offsets()
        print(tel.offsets)

    if args.focus:
        tel.get_focus()
        print(tel.focus)

    if args.weather:
        tel.get_weather()
        print(tel.weather)

    if args.hartmann:
        tel.get_hartmann()
        print(tel.hartmann)


if __name__ == '__main__':
    main()



#!/usr/bin/env python3
"""
A tool to grab a single BOSS image and pull a few items from its header. It is
 used in bin/sloan_log.py, but it could be used directly as well.
"""
import argparse
from pathlib import Path
from astropy.time import Time
import fitsio


class BOSSRaw:
    """A class to parse raw data from APOGEE. The purpose of collecting this
    raw data is to future-proof things that need these ouptuts in case
    things like autoschedulers change, which many libraries depend on. This
    will hopefully help SDSS-V logging"""

    def __init__(self, fil):
        self.fil = fil
        header = fitsio.read_header(fil)

        self.dither = header['MGDPOS']
        if not self.dither:  # This key started working instead during SDSS-V
            self.dither = header['POINTING'][0]
        self.exp_time = int(header['EXPTIME'])
        self.isot = Time(header['DATE-OBS'])  # UTC
        self.plate_id = header['PLATEID']
        self.cart_id = header['CARTID']
        self.exp_id = int(str(fil).split('-')[-1].split('.')[0])
        self.lead = header['PLATETYP']
        if 'Closed' in header['HARTMANN']:
            self.hartmann = 'Closed'
            self.flavor = header['FLAVOR'].capitalize()
        elif 'Out' in header['HARTMANN']:
            self.hartmann = 'Open'
            self.flavor = header['FLAVOR'].capitalize()
            self.hart_resids = []
        else:
            self.hartmann = header['HARTMANN']
            self.flavor = 'Hart'

        # self.seeing = header['SEEING']
        # self.img_type = header['IMAGETYP']


def main():
    parser = argparse.ArgumentParser()
    parser.add_argument('-t', '--today', action='store_true')
    args = parser.parse_args()
    parser.add_argument('-m', '--mjd',
                        help='If not today (-t), the mjd to search')
    parser.add_argument('-v', '--verbose', action='count', default=1,
                        help='Show details, can be stacked')
    if args.today:
        mjd_today = int(Time.now().sjd)
        data_dir = '/data/spectro/{}/'.format(mjd_today)
    elif args.mjd:
        data_dir = '/data/spectro/{}/'.format(args.mjd)
    else:
        raise Exception('No date specified')

    for path in Path(data_dir).rglob('sdR*.fit.gz'):
        print(path)


if __name__ == '__main__':
    main()



#!/usr/bin/env python

"""
mount disk to read data from sdss-hub

May (SJD 58240) - (SJD 58270)
April 58210 -- 58239
mjd 58271 - data with boss, apogee, manga

Examples:
timeTrack.py -a -m1 58268 -m2 58269 -v
timetrack.py -a -m1 58268 -m2 58269
timetrack.py -a -m1 58240 -m2 58270 -v
timetrack.py -b -v
timetrack.py -b -m1 58240 -m2 58270 -v

timetrack.py -b -a -m  -m1 58271 -m2 58271

"""

import argparse
from bin import sjd
# import warnings
from pathlib import Path
import fitsio
import tqdm
from bs4 import BeautifulSoup
import sys
import textwrap
from typing import Union, Tuple

# warnings.filterwarnings('ignore')
sys.setrecursionlimit(10000)  # This is a very dangerous operation that


# BeautifulSoup seems to need for many SOS pages. It needs to recursively look
# through the whole html file to make a fancy python object out of it. If this
# number is too high, it could easily trigger a segmentation fault


class Plate:
    def __init__(self, plate_id: int, lead_survey: str, cart: int):
        self.plate_id = plate_id
        self.lead_survey = lead_survey
        self.cart = cart
        self.b_count = 0
        self.a_count = 0

    def __str__(self):
        return (f'{self.plate_id:5}, {self.lead_survey:11},'
                f' {self.b_count:3} BOSS, {self.a_count:3} APOGEE')

    def __eq__(self, other: str):
        return other == self.lead_survey


def summarize(mission_name: str, mission_key: Union[str, Tuple[str, str]],
              plates: dict, verbose: bool = False):
    print('=' * 80)
    print(f'{mission_name:^80}')
    print('=' * 80)
    b_tot = 0
    a_tot = 0
    plate_ids = []
    for plate_id in sorted(plates.keys()):
        plate = plates[plate_id]
        condition = (mission_key == plate if isinstance(mission_key, str)
                     else any([key == plate for key in mission_key]))
        if condition:
            if verbose:
                print(plate)
            plate_ids.append(f'{plate_id:.0f}')
            b_tot += plate.b_count
            a_tot += plate.a_count
    print(textwrap.fill(', '.join(plate_ids), width=80))
    print(f'Total BOSS exposures: {b_tot}\nTotal APOGEE Exposures:'
          f' {a_tot}\nPlates Visited: {len(plate_ids)}\n')
    return b_tot, a_tot


def parse_args():
    desc = 'Creates a report of all observed plates for time tracking'
    parser = argparse.ArgumentParser(description=desc)
    parser.add_argument('--m1', help='start mjd, default current mjd',
                        default=sjd.sjd(), type=int)
    parser.add_argument('--m2', help='end mjd, default current mjd',
                        default=sjd.sjd(), type=int)
    parser.add_argument('-a', '--apogee', help='get APOGEE-2 report',
                        action="store_true")
    parser.add_argument('-b', '--bhm', help='get BHM report',
                        action="store_true")
    parser.add_argument('-e', '--eboss', help='get eBOSS report',
                        action="store_true")
    parser.add_argument('-m', '--mwm', help='get MWM report',
                        action="store_true")
    parser.add_argument('--manga', help='get MaNGA report',
                        action="store_true")
    parser.add_argument('-v', '--verbose', help='verbose data',
                        action="store_true")
    parser.add_argument('-f', '--force', action='store_true',
                        help='Forces analysis to conitinue even if a file is'
                             ' missing from /data or ~/data')

    args = parser.parse_args()
    return args


def main(args=parse_args()):
    if not any((args.apogee, args.bhm, args.mwm, args.eboss, args.manga)):
        raise args.error('No mission arguments given, nothing to do')
    plates = {}
    for mjd in tqdm.tqdm(list(range(args.m1, args.m2 + 1))):

        # APOGEE QL Check
        qr_path = Path(f'/data/apogee/quickred/{mjd}/')
        # The quickred image needs to exist. If you're on a Mac using Catalina,
        # you can't make /data, so a ~/data directory with 1D quickreds will
        # work.
        if not qr_path.exists():
            qr_path = Path.home() / f'data/apogee/quickred/{mjd}/'
            if not qr_path.exists():
                if not args.force:
                    raise FileNotFoundError('No path to apogee quickred files'
                                            ' found, cannot build plate list\n'
                                            f'{qr_path.as_posix()}')
                else:
                    print(f'Failed to parse APOGEE quickred on {mjd} with'
                          f' plate {qr_path}')
                    continue
        # Checks each 1D-a- image for plate info, and add it to the count if
        # it's a science image
        for fits in qr_path.glob('ap1D-a-*.fits.fz'):
            header = fitsio.read_header(fits.as_posix())
            if header['EXPTYPE'] == 'OBJECT':
                if header['PLATEID'] in plates.keys():
                    # If a BHM plate is observed with a MWM bypass, then it will
                    # be saved with SRVYMODE="MWM lead". To fix this, I need
                    # to see if they match, and if they don't check to see if
                    # either is BHM, and if so, replace the lead with BHM.
                    # This won't be an issue if glob finds a non-bypassed plate
                    # first, which is kinda random, so it doesn't show up all
                    # the time.
                    if plates[header['PLATEID']] != header['SRVYMODE']:
                        if ((header['SRVYMODE'] == 'BHM lead')
                                or (plates[header["PLATEID"]] == "BHM lead")):
                            plates[header["PLATEID"]].lead_survey = "BHM lead"

                    plates[header["PLATEID"]].a_count += 1
                else:
                    if header['SRVYMODE'] != 'None':
                        plates[header['PLATEID']] = Plate(header['PLATEID'],
                                                          header['SRVYMODE'],
                                                          header['CARTID'])
                        plates[header['PLATEID']].a_count += 1
                    else:  # APOGEE-2
                        plates[header['PLATEID']] = Plate(header['PLATEID'],
                                                          header['PLATETYP'],
                                                          header['CARTID'])
                        plates[header['PLATEID']].a_count += 1

        # SOS Check
        sos_path = Path(f'/data/boss/sos/{mjd}/logfile-{mjd}.html')
        if not sos_path.exists():
            sos_path = Path(f'/data/manga/dos/{mjd}/web/logfile-{mjd}.html')
            if not sos_path.exists():
                sos_path = Path(f'~/data/boss/sos/{mjd}/logfile-{mjd}.html')
                if not sos_path.exists():
                    if not args.force:
                        raise FileNotFoundError('No path to SOS file'
                                                ' found, cannot build plate'
                                                f' list\n{sos_path.as_posix()}')
                    else:
                        continue
        sos_soup = BeautifulSoup(sos_path.open('r').read(), 'html.parser')
        for plate in sos_soup.find_all('caption'):
            plate_id = int(plate.find('b').decode().split()[2])
            try:
                for line in plate.find('tr').decode().split('<tr>'):
                    if '>(S/N)^2' in line:
                        plates[plate_id].b_count += 1
            except AttributeError:
                print(f'Failed to parse SOS on {mjd} with plate {plate_id}')
                continue
    if args.verbose:
        print('Plates Data')
        for plate in plates.values():
            print(plate)
        print()
    b_big_total = 0
    a_big_total = 0

    if args.manga:
        b, a = summarize('MaNGA', 'MaNGA dither', plates, args.verbose)
        b_big_total += b
        a_big_total += a

    if args.apogee:
        b, a = summarize('APOGEE-2', ('APOGEE lead', 'APOGEE-2'), plates,
                         args.verbose)
        b_big_total += b
        a_big_total += a

    if args.mwm:
        b, a = summarize('Milky Way Mapper', 'MWM lead', plates, args.verbose)
        b_big_total += b
        a_big_total += a

    if args.bhm:
        b, a = summarize('Black Hole Mapper', 'BHM lead', plates, args.verbose)
        b_big_total += b
        a_big_total += a

    print('=' * 80)
    print(f'{"Overall":^80}')
    print('=' * 80)

    print(f'Total BOSS exposures: {b_big_total}\nTotal APOGEE Exposures:'
          f' {a_big_total}\nTotal Plates Visited: {len(plates)}')

    return 0


if __name__ == "__main__":
    main()



#!/usr/bin/env python3
import unittest
from pathlib import Path
import sloan_log, sjd


class TestSloanLog(unittest.TestCase):

    def test_directory(self):
        """Checks to see if the directory for new data is available to this
        computer"""
        ap_data_dir = sloan_log.ap_dir
        b_data_dir = sloan_log.b_dir
        print(ap_data_dir, b_data_dir)
        self.assertTrue(Path(ap_data_dir).exists())
        self.assertTrue(Path(b_data_dir).exists())

    def setUp(self) -> None:

        class Dummy(object):
            pass

        self.args = Dummy()

    def test_known_data(self):
        """Runs on an old dataset that I know used to run successfully"""

        self.args.sjd = 59011
        self.args.print = True
        self.args.summary = True
        self.args.data = True
        self.args.boss = True
        self.args.apogee = True
        self.args.p_boss = True
        self.args.p_apogee = True
        self.args.log_support = True
        self.args.morning = False
        self.args.verbose = True
        self.args.noprogress = False
        self.args.mirrors = True
        self.args.telstatus = True
        self.args.legacy_aptest = False

        ap_data_dir = sloan_log.ap_dir / '{}'.format(self.args.sjd)
        b_data_dir = sloan_log.b_dir / '{}'.format(self.args.sjd)
        ap_images = list(Path(ap_data_dir).glob('apR-a*.apz'))
        b_images = list(Path(b_data_dir).glob('sdR-r1*fit.gz'))
        log = sloan_log.Logging(ap_images, b_images, self.args)
        log.parse_images()
        log.sort()
        log.count_dithers()
        log.p_summary()
        log.p_data()
        log.p_boss()
        log.p_apogee()
        log.log_support()

    def test_known_data_boss(self):
        """Runs on an old dataset that I know used to run successfully, this one
        only runs BOSS data summary
        """

        self.args.sjd = 59011
        self.args.print = False
        self.args.summary = False
        self.args.data = False
        self.args.boss = True
        self.args.apogee = False
        self.args.p_boss = True
        self.args.p_apogee = False
        self.args.log_support = False
        self.args.morning = False
        self.args.verbose = True
        self.args.mirrors = False
        self.args.telstatus = False
        self.args.legacy_aptest = False

        ap_data_dir = sloan_log.ap_dir / '{}'.format(self.args.sjd)
        b_data_dir = sloan_log.b_dir / '{}'.format(self.args.sjd)
        ap_images = list(Path(ap_data_dir).glob('apR-a*.apz'))
        b_images = list(Path(b_data_dir).glob('sdR-r1*fit.gz'))
        log = sloan_log.Logging(ap_images, b_images, self.args)
        log.parse_images()
        log.sort()
        log.count_dithers()
        log.p_boss()

    def test_known_data_apogee(self):
        """Runs on an old dataset that I know used to run successfully, this one
        only checks APOGEE and prints its summary
        """

        self.args.sjd = 59011
        self.args.print = False
        self.args.summary = False
        self.args.data = False
        self.args.boss = False
        self.args.apogee = True
        self.args.p_boss = False
        self.args.p_apogee = True
        self.args.log_support = False
        self.args.morning = False
        self.args.verbose = True
        self.args.morrors = False
        self.args.telstatus = False
        self.args.legacy_aptest = False

        ap_data_dir = sloan_log.ap_dir / '{}'.format(self.args.sjd)
        b_data_dir = sloan_log.b_dir / '{}'.format(self.args.sjd)
        ap_images = list(Path(ap_data_dir).glob('apR-a*.apz'))
        b_images = list(Path(b_data_dir).glob('sdR-r1*fit.gz'))
        log = sloan_log.Logging(ap_images, b_images, self.args)
        log.parse_images()
        log.sort()
        log.count_dithers()
        log.p_apogee()

    def test_log_support(self):
        """Runs on an old dataset that I know used to run successfully"""

        self.args.sjd = 59011
        self.args.print = False
        self.args.summary = False
        self.args.data = False
        self.args.boss = False
        self.args.apogee = False
        self.args.p_boss = False
        self.args.p_apogee = False
        self.args.log_support = True
        self.args.morning = False
        self.args.verbose = True
        self.args.mirrors = False
        self.args.telstatus = False
        self.args.legacy_aptest = False

        ap_data_dir = sloan_log.ap_dir / '{}'.format(self.args.sjd)
        b_data_dir = sloan_log.b_dir / '{}'.format(self.args.sjd)
        ap_images = list(Path(ap_data_dir).glob('apR-a*.apz'))
        b_images = list(Path(b_data_dir).glob('sdR-r1*fit.gz'))
        log = sloan_log.Logging(ap_images, b_images, self.args)
        log.parse_images()
        log.sort()
        log.count_dithers()
        log.log_support()

    def test_log_support_today(self):
        """Runs on an old dataset that I know used to run successfully"""

        self.args.sjd = sjd.sjd()
        self.args.print = False
        self.args.summary = False
        self.args.data = False
        self.args.boss = False
        self.args.apogee = False
        self.args.p_boss = False
        self.args.p_apogee = False
        self.args.log_support = True
        self.args.morning = False
        self.args.verbose = True
        self.args.mirrors = False
        self.args.telstatus = False
        self.args.legacy_aptest = False

        ap_data_dir = sloan_log.ap_dir / '{}'.format(self.args.sjd)
        b_data_dir = sloan_log.b_dir / '{}'.format(self.args.sjd)
        ap_images = list(Path(ap_data_dir).glob('apR-a*.apz'))
        b_images = list(Path(b_data_dir).glob('sdR-r1*fit.gz'))
        log = sloan_log.Logging(ap_images, b_images, self.args)
        log.parse_images()
        log.sort()
        log.count_dithers()
        log.log_support()

    def test_mirror_numbers(self):
        """Runs on an old dataset that I know used to run successfully"""

        self.args.sjd = sjd.sjd()
        self.args.print = False
        self.args.summary = False
        self.args.data = False
        self.args.boss = False
        self.args.apogee = False
        self.args.p_boss = False
        self.args.p_apogee = False
        self.args.log_support = False
        self.args.morning = False
        self.args.verbose = True
        self.args.mirrors = True
        self.args.telstatus = False
        self.args.legacy_aptest = False

        ap_data_dir = sloan_log.ap_dir / '{}'.format(self.args.sjd)
        b_data_dir = sloan_log.b_dir / '{}'.format(self.args.sjd)
        ap_images = list(Path(ap_data_dir).glob('apR-a*.apz'))
        b_images = list(Path(b_data_dir).glob('sdR-r1*fit.gz'))
        log = sloan_log.Logging(ap_images, b_images, self.args)
        log.parse_images()
        log.sort()
        log.count_dithers()
        log.log_support()
        log.mirror_numbers()


def test_tel_status(self):
    """Test telstatus integration"""

    self.args.sjd = sjd.sjd()
    self.args.print = False
    self.args.summary = False
    self.args.data = False
    self.args.boss = False
    self.args.apogee = False
    self.args.p_boss = False
    self.args.p_apogee = False
    self.args.log_support = False
    self.args.morning = False
    self.args.verbose = True
    self.args.mirrors = False
    self.args.telstatus = True
    self.args.legacy_aptest = False

    ap_data_dir = sloan_log.ap_dir / '{}'.format(self.args.sjd)
    b_data_dir = sloan_log.b_dir / '{}'.format(self.args.sjd)
    ap_images = list(Path(ap_data_dir).glob('apR-a*.apz'))
    b_images = list(Path(b_data_dir).glob('sdR-r1*fit.gz'))
    log = sloan_log.Logging(ap_images, b_images, self.args)
    log.parse_images()
    log.sort()
    log.count_dithers()
    log.log_support()


def test_legacy_aptest(self):
    """Test utr_cdr-based aptest that matches the old aptest script"""

    class Dummy(object):
        pass

    args = Dummy()
    self.args.sjd = sjd.sjd()
    self.args.print = False
    self.args.summary = False
    self.args.data = True
    self.args.boss = False
    self.args.apogee = False
    self.args.p_boss = False
    self.args.p_apogee = False
    self.args.log_support = False
    self.args.morning = False
    self.args.verbose = True
    self.args.mirrors = False
    self.args.telstatus = False
    self.args.legacy_aptest = True

    ap_data_dir = sloan_log.ap_dir / '{}'.format(self.args.sjd)
    b_data_dir = sloan_log.b_dir / '{}'.format(self.args.sjd)
    ap_images = list(Path(ap_data_dir).glob('apR-a*.apz'))
    b_images = list(Path(b_data_dir).glob('sdR-r1*fit.gz'))
    log = sloan_log.Logging(ap_images, b_images, self.args)
    log.parse_images()
    log.sort()
    log.count_dithers()
    log.p_data()


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python3
"""
Written by Elena, rewritten by Dylan. This script gets the dust counts in a
 night and prints it

Changelog:
2020-06-20  DG  Moving main contents into a function for import functionality
 """
import argparse
import datetime
from astropy.time import Time
import numpy as np
from bin import sjd, epics_fetch

__version__ = '3.2.2'

telemetry = epics_fetch.telemetry

# TAI_UTC =34;
TAI_UTC = 0
aSjd = 40587.3
bSjd = 86400.0


def get_time_stamps(mjd):
    startStamp = sjd.sjd_to_time(int(mjd))
    endStamp = sjd.sjd_to_time(int(mjd + 1))
    start = datetime.datetime.fromtimestamp(startStamp)
    end = datetime.datetime.fromtimestamp(endStamp)
    return start, end


def parse_args():
    today = sjd.sjd()
    desc = ('Integrates the dust accumulated in a night using TRON and prints'
            ' it. Needs no args for the current date.')
    parser = argparse.ArgumentParser(description=desc)
    parser.add_argument('-m', '--mjd', help='enter mjd, default is current mjd',
                        default=today, type=int)
    parser.add_argument('-v', '--verbose', action="store_true",
                        help='print incremental dust data')
    args = parser.parse_args()
    return args


def get_dust(mjd, args):
    start, end = get_time_stamps(mjd)
    if args.verbose:
        print("mjd= {}".format(mjd))
        print("MJD start/end times")
        print(start)
        print(end)
    dust_data = telemetry.get('25m:apo:dustb', start, end, interpolation='raw',
                              scan_archives=False)
    enclosure_data = telemetry.get('25m:apo:encl25m', start, end,
                                   interpolation='raw', scan_archives=False)
    if args.verbose:
        print("         Enlosure open/close times")
        print(enclosure_data)
        print("")
    dust_data.times = Time(dust_data.times)
    dust_data.values = np.array(dust_data.values)
    enclosure_data.times = Time(enclosure_data.times)
    enclosure_data.values = np.array(enclosure_data.values)
    dust_sum = 0
    if args.verbose:
        print("date   time    dust    encl    sum")
    for t, d in zip(dust_data.times, dust_data.values):
        enclosure_state = enclosure_data.values[enclosure_data.times < t][-1]
        if enclosure_state > 0:
            dust_sum = dust_sum + d * 5 / 60.0
        if args.verbose:
            print("{} {:7.0f}    {:.0f}    {:10.0f}"
                  "".format(t, d, enclosure_state, dust_sum))
    return dust_sum


def main():
    args = parse_args()
    dust_sum = get_dust(args.mjd, args)
    print("Integrated Dust Counts: ~{:<5.0f}dust-hrs".format(
          dust_sum - dust_sum % 100))


if __name__ == '__main__':
    main()



#!/usr/bin/env python3
import unittest
from pathlib import Path
import telescope_status
import signal


class TestTelStatus(unittest.TestCase):
    def setUp(self):
        self.file = (Path(__file__).absolute().parent.parent
                     / 'bin/telescope_status.py')

    @staticmethod
    def handler(signum, frame):
        print('Exiting call')
        raise TimeoutError('The function reached timeout without other errors')

    # Problematic test that hangs, should investigate further
    # def test_print(self):
    #     signal.signal(signal.SIGALRM, self.handler)
    #     signal.alarm(10)
    #     telescope_status.main()


if __name__ == '__main__':
    unittest.main()
